"""
pip install pymupdf
"""
import os
from typing import Optional

from pydantic import BaseModel
from reportlab.lib.pagesizes import letter
from reportlab.lib.utils import ImageReader
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfgen import canvas
from reportlab.pdfgen.canvas import Canvas
# PyMuPDF 的导入名称
import fitz


class PdfTextItem(BaseModel):
    x: int
    y: int
    text: str


class PdfImageItem(BaseModel):
    class Config:
        arbitrary_types_allowed = True

    x: int
    y: int
    image: ImageReader
    image_width: Optional[int] = None
    image_height: Optional[int] = None


def create_canvas(pdf_file: str, page_size: tuple[float, float] = letter, bottom_up: int = 0) -> Canvas:
    """创建一个PDF画布，默认设置页面大小为letter，并且横放"""
    return canvas.Canvas(pdf_file, pagesize=page_size, bottomup=bottom_up)


def save_canvas(pdf_canvas: Canvas):
    """保存 Canvas """
    pdf_canvas.save()


def create_new_page(pdf_canvas: Canvas, item_list: list[PdfTextItem | PdfImageItem],
                    ttf_file: str = r'C:\Windows\Fonts\SimHei.ttf',
                    stroke_color_rgb: tuple[float, float, float] = (0.2, 0.5, 0.3),
                    fill_color_rgb: tuple[float, float, float] = (0, 0, 0),
                    font_size: int = 12,
                    line_height: int = 20,
                    is_end: bool = False):
    """
    添加内容
    :param pdf_canvas:
    :param item_list:
    :param ttf_file:
    :param stroke_color_rgb:
    :param fill_color_rgb:
    :param font_size:
    :param line_height: 行高
    :param is_end：是否是结尾页
    :return:
    """
    # 注册中文字体
    pdfmetrics.registerFont(TTFont('SimHei', ttf_file))
    pdf_canvas.setStrokeColorRGB(*stroke_color_rgb)
    pdf_canvas.setFillColorRGB(*fill_color_rgb)
    # 设置字体和字号
    pdf_canvas.setFont('SimHei', font_size)
    space_x, space_y = 0, 0
    for tmp_item in item_list:
        if isinstance(tmp_item, PdfImageItem):
            # 图片大小
            img_width, img_height = tmp_item.image.getSize()
            if tmp_item.image_width:
                img_width = tmp_item.image_width
            if tmp_item.image_height:
                img_height = tmp_item.image_height
            # 将图片添加到PDF中，指定位置和大小
            pdf_canvas.drawImage(tmp_item.image, tmp_item.x, tmp_item.y, width=img_width, height=img_height)
        elif isinstance(tmp_item, PdfTextItem):
            if tmp_item.x >= letter[0]:
                space_x = space_x + letter[0]
                tmp_item.x = tmp_item.x - space_x + 10
                tmp_item.y = tmp_item.y + font_size
            if tmp_item.y >= letter[1]:
                if space_y + letter[1] <= tmp_item.y + line_height * 2:
                    space_y = space_y + letter[1]
                    pdf_canvas.showPage()
                    pdf_canvas.setStrokeColorRGB(*stroke_color_rgb)
                    pdf_canvas.setFillColorRGB(*fill_color_rgb)
                    # 设置字体和字号
                    pdf_canvas.setFont('SimHei', font_size)
                while tmp_item.y - space_y - line_height < 0:
                    space_y = space_y - line_height
                tmp_item.y = tmp_item.y - space_y + line_height
            tmp_item.text = tmp_item.text.replace('\t', '    ')
            # 添加文字
            pdf_canvas.drawString(tmp_item.x, tmp_item.y, tmp_item.text)
    if not is_end:
        # 显示当前页面并准备下一页（以此类推可以添加更多页）
        pdf_canvas.showPage()
    return pdf_canvas


def merger(input_pdf_list: list[str], out_pdf: str, start_page_num:int=0) -> str:
    """
    合并pdf
    :param input_pdf_list: 需要合并的pdf文件列表
    :param out_pdf: 合并后的pdf
    :param start_page_num: 起始页码
    :return: 合并后的pdf
    """

    # 创建一个新的 PDF 文档对象
    merged_pdf = fitz.open()
    toc_list = []
    # 遍历要合并的 PDF 文件列表
    for input_pdf in input_pdf_list:
        # 打开当前 PDF 文件
        with fitz.open(input_pdf) as src_pdf:
            # 获取目录
            tmp_toc = src_pdf.get_toc()
            for entry in tmp_toc:
                # 目录级别
                level = entry[0]
                # 目录标题
                title = entry[1]
                # 目录所在页码
                page_number = entry[2]

                # [页码, 标题， 目录level]
                toc_list.append([level, title, start_page_num+page_number])
            # 添加到新文档的末尾
            merged_pdf.insert_pdf(src_pdf)
            start_page_num = start_page_num + src_pdf.page_count

    merged_pdf.set_toc(toc_list)
    # 保存合并后的 PDF 文件
    merged_pdf.save(out_pdf)

    # 关闭合并后的 PDF 文档对象
    merged_pdf.close()
    return out_pdf

if __name__ == '__main__':
    root_path = os.path.dirname(__file__)
    cover_pdf = os.path.join(root_path, 'cover.pdf')
    pdf_canvas = create_canvas(cover_pdf)
    # 加载图片
    img = ImageReader('input/cover.jpg')
    image_width, image_height = letter
    create_new_page(pdf_canvas,
                              item_list=[PdfImageItem(x=0, y=0, image=img,
                                                                image_width=image_width,
                                                                image_height=image_height)], is_end=True)


    # line_list = file_tools.read_file_lines(os.path.join(os.path.dirname(__file__), 'input/目录.TXT'))
    #
    # pdf_tools.create_new_page(pdf_canvas, item_list=[pdf_tools.PdfTextItem(x=50, y=20 * (index + 1), text=tmp_line)
    #                                                  for index, tmp_line in enumerate(line_list)])
    save_canvas(pdf_canvas)

    pdf_list = [cover_pdf]
    # pdf文件存放路径
    input_path = os.path.join(root_path, 'input')
    # 将待拼接的pdf文件以绝对路径的形式放在一个列表里
    pdf_list.extend(sorted([os.path.join(input_path, f) for f in os.listdir(input_path) if f.lower().endswith('.pdf')]))
    merger(pdf_list, 'output/LINUX与UNIX Shell编程指南.pdf', start_page_num=0)


from typing import Any

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService
from afeng_tools.fastapi_tool.common.service import icon_base_service
from afeng_tools.pydantic_tool.model.common_models import LinkItem
from afeng_tools.sqlalchemy_tools.crdu import base_crdu


class CategoryPoService(PoService):
    """
    使用示例：category_po_service = CategoryPoService(app_info.db_code, CategoryInfoPo)
    """
    _table_name_ = "tb_category_info"

    def get_category(self, title: str, group_code: int, parent_code: int = None) -> Any:
        category_po = self.get(self.model_type.title == title,
                               self.model_type.group_code == group_code, self.model_type.parent_code == parent_code)
        if not category_po:
            category_po = self.save(self.model_type(
                group_code=group_code,
                parent_code=parent_code,
                title=title
            ), auto_code=True)
        return category_po

    def query_all_data(self, group_code: int = None) -> list:
        """查询所有数据"""
        if group_code:
            return base_crdu.query_all(self.model_type, self.model_type.group_code == group_code, db_code=self.db_code)
        else:
            return base_crdu.query_all(self.model_type, db_code=self.db_code)

    def query_all_data_dict(self, group_code: int = None) -> dict[str, Any]:
        """
        查询所有数据字典(code为键)
        :return: {code: BookCategoryPo}
        """
        return {tmp.code: tmp for tmp in self.query_all_data(group_code)}

    def query_group_data_dict(self) -> dict[str, list[Any]]:
        """
        查询分组数据字典(group_code为键)
        :return: {group_code: BookCategoryPo}
        """
        data_list = self.query_all_data()
        result_dict = dict()
        for tmp in data_list:
            if not result_dict.get(tmp.group_code):
                result_dict[tmp.group_code] = []
            result_dict[tmp.group_code].append(tmp)
        return result_dict

    def query_group_data(self, group_code: int,
                         category_dict: dict[int, list[Any]] = None) -> list[Any]:
        """
        查询某个分组的类别数据
        :param group_code: 分组编码
        :param category_dict: 类别字典：{group_code: BookCategoryPo}
        :return:
        """
        if category_dict:
            return category_dict.get(group_code)
        else:
            return base_crdu.query_all(self.model_type, self.model_type.group_code == group_code,
                                       db_code=self.db_code)

    def get_by_code(self, group_code: str, code: str) -> Any:
        return self.get(self.model_type.group_code == group_code, self.model_type.code == code)

    @classmethod
    def convert_to_link_item(cls, category_po, is_active: bool = False,
                             category_href_prefix: str = '/category') -> LinkItem:
        return LinkItem(
            target='_blank',
            title=category_po.title,
            href=f'{category_href_prefix}/{category_po.code}',
            code=category_po.code,
            description=category_po.description,
            image=icon_base_service.get_icon_code(icon_type=category_po.icon_type,
                                                  icon_value=category_po.icon_value,
                                                  alt=category_po.title,
                                                  image_src=category_po.image_src),
            is_active=is_active
        )



import re


def create_table(sql):
    if re.findall(r"CREATE (\w+)", sql) in ["SEQUENCE", "TABLE"]:
        pass
    raise NotImplementedError


def drop_table(sql):
    if re.findall(r"DROP (\w+)", sql) in ["SEQUENCE", "TABLE"]:
        pass
    raise NotImplementedError





from afeng_tools.fastapi_tool.common.po_service.article_po_service_ import ArticlePoService, ArticleDetailPoService
from afeng_tools.fastapi_tool.common.po_service.blacklist_po_service_ import BlacklistPoService
from afeng_tools.fastapi_tool.common.po_service.category_po_service_ import CategoryPoService
from afeng_tools.fastapi_tool.common.po_service.group_po_service_ import GroupPoService
from afeng_tools.fastapi_tool.common.po_service.link_po_service_ import LinkPoService
from afeng_tools.fastapi_tool.common.po_service.relation_po_service_ import RelationPoService
from afeng_tools.fastapi_tool.common.po_service.resource_po_service_ import ResourcePoService
from afeng_tools.fastapi_tool.common.po_service.sitemap_po_service_ import SitemapPoService
from afeng_tools.fastapi_tool.common.po_service.tag_po_service_ import TagPoService, TagRelationPoService
from afeng_tools.fastapi_tool.common.po_service.tmp_po_service_ import TmpSortPoService


class CommonPoService:
    """通用po服务"""

    def __init__(self, db_code: str):
        self.db_code = db_code

        self._group_po_service = None
        self._category_po_service = None
        self._link_po_service = None
        self._resource_po_service = None
        self._tag_po_service = None
        self._tag_relation_po_service = None
        self._article_po_service = None
        self._article_detail_po_service = None
        self._relation_po_service = None
        self._blacklist_po_service = None
        self._tmp_sort_po_service = None
        self._sitemap_po_service = None

    @property
    def group_po_service(self) -> GroupPoService:
        if self._group_po_service is None:
            self._group_po_service = GroupPoService(self.db_code)
        return self._group_po_service

    @property
    def category_po_service(self) -> CategoryPoService:
        if self._category_po_service is None:
            self._category_po_service = CategoryPoService(self.db_code)
        return self._category_po_service

    @property
    def link_po_service(self) -> LinkPoService:
        if self._link_po_service is None:
            self._link_po_service = LinkPoService(self.db_code)
        return self._link_po_service

    @property
    def resource_po_service(self) -> ResourcePoService:
        if self._resource_po_service is None:
            self._resource_po_service = ResourcePoService(self.db_code)
        return self._resource_po_service

    @property
    def tag_po_service(self) -> TagPoService:
        if self._tag_po_service is None:
            self._tag_po_service = TagPoService(self.db_code)
        return self._tag_po_service

    @property
    def tag_relation_po_service(self) -> TagRelationPoService:
        if self._tag_relation_po_service is None:
            self._tag_relation_po_service = TagRelationPoService(self.db_code)
        return self._tag_relation_po_service

    @property
    def article_po_service(self) -> ArticlePoService:
        if self._article_po_service is None:
            self._article_po_service = ArticlePoService(self.db_code)
        return self._article_po_service

    @property
    def article_detail_po_service(self) -> ArticleDetailPoService:
        if self._article_detail_po_service is None:
            self._article_detail_po_service = ArticleDetailPoService(self.db_code)
        return self._article_detail_po_service

    @property
    def relation_po_service(self) -> RelationPoService:
        if self._relation_po_service is None:
            self._relation_po_service = RelationPoService(self.db_code)
        return self._relation_po_service

    @property
    def blacklist_po_service(self) -> BlacklistPoService:
        if self._blacklist_po_service is None:
            self._blacklist_po_service = BlacklistPoService(self.db_code)
        return self._blacklist_po_service

    @property
    def tmp_sort_po_service(self) -> TmpSortPoService:
        if self._tmp_sort_po_service is None:
            self._tmp_sort_po_service = TmpSortPoService(self.db_code)
        return self._tmp_sort_po_service

    @property
    def sitemap_po_service(self) -> SitemapPoService:
        if self._sitemap_po_service is None:
            self._sitemap_po_service = SitemapPoService(self.db_code)
        return self._sitemap_po_service



"""
配置工具类
"""
import logging
import os
import tempfile
from datetime import datetime
from typing import Any

from afeng_tools.application_tool import application_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum, SettingsStaticItem
from afeng_tools.python_tool import dict_tools

# 程序配置
APP_CONFIG_CACHE = {}
# 程序配置参数，如：{root}代表项目根目录
APP_CONFIG_PARAMS = {}


def init_config_param(**kwargs) -> dict:
    """
    初始化配置参数， 如：init_config_param(root=ROOT_PATH, resource=RESOURCE_PATH)
    :param kwargs: 如：root=‘路径’， 则可以在yaml配置文件中使用{root}替换路径
    :return: 程序配置参数
    """
    APP_CONFIG_PARAMS.update(kwargs)
    # 用户目录
    APP_CONFIG_PARAMS['user_home'] = os.path.expanduser('~')
    # 临时目录
    APP_CONFIG_PARAMS['tmp'] = tempfile.gettempdir()
    APP_CONFIG_PARAMS['date'] = datetime.today().strftime('%Y%m%d')
    APP_CONFIG_PARAMS['datetime'] = datetime.today().strftime('%Y%m%d%H%M%S')
    APP_CONFIG_PARAMS['datetime2'] = datetime.today().strftime('%Y%m%d%H%M%S%f')
    APP_CONFIG_PARAMS['app_code'] = 'afeng'
    return kwargs


def _format_param(config_value: Any):
    """格式化参数，如：替换配置中的{root}为根路径"""
    if config_value is not None and isinstance(config_value, str):
        for tmp_key in APP_CONFIG_PARAMS:
            if '{' + tmp_key + '}' in config_value:
                config_value = config_value.replace('{' + tmp_key + '}', str(APP_CONFIG_PARAMS.get(tmp_key)))
    return config_value


def _get_config(config_dict: dict, key_enum: SettingsKeyEnum):
    if '.' in key_enum.value.name:
        return dict_tools.tree_get_value(config_dict, key_enum.value.name.split('.'))
    return config_dict.get(key_enum.value.name)


def _init_static_config(config_dict: dict):
    """初始化静态文件配置"""
    app_static_value = config_dict.get(SettingsKeyEnum.server_static.value.name)
    if app_static_value:
        if isinstance(app_static_value, list):
            app_static_value = [SettingsStaticItem(name=tmp.get('name'), url=tmp.get('url'),
                                                   path=_format_param(tmp.get('path')))
                                for tmp in app_static_value]
        elif isinstance(app_static_value, dict):
            app_static_value = SettingsStaticItem(name=app_static_value.get('name'),
                                                  url=app_static_value.get('url'),
                                                  path=_format_param(app_static_value.get('path')))
        config_dict[SettingsKeyEnum.server_static.value.name] = app_static_value
    return config_dict


def _check_config(config_dict: dict, is_reload: bool = False) -> bool:
    """检查配置：用于检查配置项是否有问题"""
    error_msg_list = []
    is_web = _get_config(config_dict, SettingsKeyEnum.app_is_web)
    for tmp_enum in SettingsKeyEnum.__iter__():
        if (not is_web and tmp_enum.name.startswith('server_')) or tmp_enum == SettingsKeyEnum.server_static:
            continue
        settings_key_item = tmp_enum.value
        config_value = _get_config(config_dict, tmp_enum)
        if settings_key_item.is_required:
            if config_value is None:
                error_msg_list.append(f"缺少必要的配置参数[{settings_key_item.name}], "
                                      f"该参数用于配置[{settings_key_item.comment}]")
        if _get_config(config_dict, tmp_enum) is not None:

            if not isinstance(config_value, settings_key_item.value_type):
                error_msg_list.append(
                    f"配置参数[{settings_key_item.name}]的值类型不对，需要的值类型是{settings_key_item.value_type}, "
                    f"现在配置的值类型是[{type(config_value)}]")
    if error_msg_list:
        error_msg = "\t\n".join(error_msg_list)
        if is_reload:
            logging.error(f'配置文件配置有问题：\n\t{error_msg}')
            return False
        raise ValueError(f'配置文件配置有问题：\n\t{error_msg}')
    return True


def _init_config_default_value(config_dict: dict) -> dict:
    """初始化设置配置默认值"""
    for tmp_enum in SettingsKeyEnum.__iter__():
        tmp_key_item = tmp_enum.value
        if tmp_key_item.default is not None and config_dict.get(tmp_key_item.name) is None:
            # 如果为None，设置为默认值
            config_dict[tmp_key_item.name] = tmp_key_item.default
    return config_dict


def init_load_config(yaml_file, is_reload: bool = False) -> dict:
    """
    初始化配置
    :param yaml_file: yaml配置我呢见
    :param is_reload: 是否是重新加载配置
    :return:
    """
    result = application_tools.read_all_config(yaml_file)
    # 设置配置默认值
    result = _init_config_default_value(result)
    # 格式化参数
    for tmp_key in result.keys():
        result[tmp_key] = _format_param(result.get(tmp_key))
    # 初始化静态文件配置
    result = _init_static_config(result)
    # 格式配置（组装一下上下级关系）
    result = dict_tools.format_config_dict(result)
    # 检查配置
    if _check_config(result, is_reload=is_reload):
        APP_CONFIG_CACHE.clear()
        APP_CONFIG_CACHE.update(result)
        return result
    else:
        return APP_CONFIG_CACHE


def reload_config(yaml_file) -> dict:
    """重新加载配置"""
    return init_load_config(yaml_file, is_reload=True)


def get_config(config_key: str | SettingsKeyEnum, value_is_list: bool = False) -> Any | list[Any]:
    """
    获取配置
    :param config_key: 配置的键，如：app.name
    :param value_is_list: 是是否是列表
    :return: 配置的值
    """
    if value_is_list:
        return application_tools.get_settings_item_value_list(APP_CONFIG_CACHE, config_key)
    if isinstance(config_key, SettingsKeyEnum):
        config_key = config_key.value.name
    return APP_CONFIG_CACHE.get(config_key)


def get_child_config(config_key_start: str) -> dict[str, Any]:
    """
    获取子项配置
    :param config_key_start: 配置键开头， 如： app.db_url
    :return: 匹配的子项配置
    """
    return {tmp_config_key: tmp_config_value for tmp_config_key, tmp_config_value in APP_CONFIG_CACHE.items() if
            tmp_config_key.startswith(config_key_start)}



from typing import Callable

from afeng_tools.decorator_tool.decorator_tools import run_func
from afeng_tools.log_tool import loguru_tools
from afeng_tools.log_tool.loguru_tools import log_error

from afeng_tools.sqlalchemy_tools import sqlalchemy_settings

logger = loguru_tools.get_logger()


def auto_commit_db(db_code: str = 'default', retry: int = 0):
    """自动注入自动提交的db: Session"""
    def auto_wrap(func):
        def wrap(*args, **kwargs):
            if 'db' in kwargs:
                return run_func(func, *args, **kwargs)
            tmp_db = None
            args_hava_db_code = False
            args_db_code = None
            if 'db_code' in kwargs:
                args_hava_db_code = True
                args_db_code = kwargs.pop('db_code')
                if args_db_code:
                    tmp_db = sqlalchemy_settings.get_database(db_code=args_db_code)
            if not tmp_db:
                tmp_db = sqlalchemy_settings.get_database(db_code=db_code)
            with tmp_db.get_session() as session:
                try:
                    kwargs['db'] = session
                    result = run_func(func, *args, **kwargs)
                    session.commit()
                    return result
                except Exception as e:
                    logger.error(f'Db session commit data exception, will to retry[{retry}]')
                    session.rollback()
                    if retry > 0:
                        kwargs.pop('db')
                        if args_hava_db_code:
                            kwargs['db_code'] = args_db_code
                        return auto_commit_db(db_code=db_code, retry=retry - 1)(func)(*args, **kwargs)
                    else:
                        raise e
        return wrap
    return auto_wrap


def auto_db(db_code: str = 'default', retry: int = 3):
    """自动注入db: Session"""
    def auto_wrap(func: Callable):
        def wrap(*args, **kwargs):
            if 'db' in kwargs:
                return run_func(func, *args, **kwargs)
            tmp_db = None
            args_hava_db_code = False
            args_db_code = None
            if 'db_code' in kwargs:
                args_hava_db_code = True
                args_db_code = kwargs.pop('db_code')
                if args_db_code:
                    tmp_db = sqlalchemy_settings.get_database(db_code=args_db_code)
            if not tmp_db:
                tmp_db = sqlalchemy_settings.get_database(db_code=db_code)
            with tmp_db.get_session() as session:
                kwargs['db'] = session
                try:
                    return run_func(func, *args, **kwargs)
                except Exception as e:
                    logger.error(f'Db session commit data exception, will to retry[{retry}]')
                    if retry > 0:
                        kwargs.pop('db')
                        if args_hava_db_code:
                            kwargs['db_code'] = args_db_code
                        return auto_db(db_code=db_code, retry=retry-1)(func)(*args, **kwargs)
                    else:
                        raise e
        return wrap

    return auto_wrap


def auto_commit_conn(db_code: str = 'default'):
    """自动注入自动提交的conn: Connection"""

    def auto_wrap(func):
        def wrap(*args, **kwargs):
            if 'conn' in kwargs:
                return run_func(func, *args, **kwargs)
            tmp_db = None
            if 'db_code' in kwargs:
                session_db_code = kwargs.pop('db_code')
                if session_db_code:
                    tmp_db = sqlalchemy_settings.get_database(db_code=session_db_code)
            if not tmp_db:
                tmp_db = sqlalchemy_settings.get_database(db_code=db_code)
            with tmp_db.engine.connect() as conn:
                try:
                    kwargs['conn'] = conn
                    result = run_func(func, *args, **kwargs)
                    conn.commit()
                    return result
                except Exception as e:
                    logger.error(f'Db connection commit data exception')
                    conn.rollback()
                    raise e

        return wrap

    return auto_wrap


def auto_conn(db_code: str = 'default'):
    """自动注入conn: Connection"""

    def auto_wrap(func):
        def wrap(*args, **kwargs):
            if 'conn' in kwargs:
                return run_func(func, *args, **kwargs)
            tmp_db = None
            if 'db_code' in kwargs:
                session_db_code = kwargs.pop('db_code')
                if session_db_code:
                    tmp_db = sqlalchemy_settings.get_database(db_code=session_db_code)
            if not tmp_db:
                tmp_db = sqlalchemy_settings.get_database(db_code=db_code)
            with tmp_db.engine.connect() as conn:
                kwargs['conn'] = conn
                return run_func(func, *args, **kwargs)

        return wrap

    return auto_wrap



"""
fastapi模板工具
"""
import os
from datetime import datetime
from typing import Callable

from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum


def is_need_reload(save_template_file: str, reload_freq: ReloadFreqEnum = ReloadFreqEnum.weekly, is_debug:bool=False) -> bool:
    """
    是否刷新模板
    :param save_template_file: 保存的模板文件
    :param reload_freq: 刷新频率
    :param is_debug: 是否是debug
    :return:
    """
    if not os.path.exists(save_template_file) or reload_freq == reload_freq.always or is_debug:
        return True
    old_update_time = os.stat(save_template_file).st_mtime
    reload_freq_item = reload_freq.value
    if reload_freq_item.judge_func and isinstance(reload_freq_item.judge_func, Callable):
        return reload_freq_item.judge_func(old_update_time)
    else:
        return (datetime.now().timestamp() - reload_freq_item.seconds_value) > old_update_time



# flake8: noqa

"""
    xpan

    xpanapi  # noqa: E501

    The version of the OpenAPI document: 0.1
    Generated by: https://openapi-generator.tech
"""


__version__ = "1.0.0"

# import ApiClient
from openapi_client.api_client import ApiClient

# import Configuration
from openapi_client.configuration import Configuration

# import exceptions
from openapi_client.exceptions import OpenApiException
from openapi_client.exceptions import ApiAttributeError
from openapi_client.exceptions import ApiTypeError
from openapi_client.exceptions import ApiValueError
from openapi_client.exceptions import ApiKeyError
from openapi_client.exceptions import ApiException



import os
from typing import Callable

from aligo import Aligo, BaseShareFile, BaseFile

from afeng_tools.aliyun_pan_tool import aliyun_pan_tools
from afeng_tools.aliyun_pan_tool.aliyun_pan_share_tools import get_share_cache_token
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import UploadCreateResult, FileInfo
from afeng_tools.baidu_pan_tool.tools import baidu_pan_file_upload_tools, baidu_pan_file_meta_tools, \
    baidu_pan_file_path_tools
from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error
from afeng_tools.os_tool import os_tools

logger = get_logger()


def get_baidupan_old_file_list(baidupan_access_token: str, pan_path: str,
                               interval_seconds: float = 20) -> dict[str, FileInfo]:
    """
    获取百度网盘旧文件列表
    :param baidupan_access_token:
    :param pan_path: 网盘路径
    :return: [(文件路径，文件大小)，(文件路径，文件大小)]
    :param interval_seconds: 间隔秒数，递归查询太过频繁请求接口会报错，这里是两次请求的间隔秒数
    """
    pan_path = baidu_pan_file_path_tools.parse_pan_path(pan_path)
    old_file_list = baidu_pan_file_path_tools.list_all_file(baidupan_access_token, pan_path, recursion=1,
                                                            only_file=True, interval_seconds=interval_seconds)
    if old_file_list:
        return {tmp.path: tmp for tmp in old_file_list}


def get_baidupan_save_path(alipan_path: str, baidupan_save_path_func: Callable[[str, str], str]) -> str:
    """获取百度网盘保存路径"""
    if '/' in alipan_path:
        alipan_path_info_arr = alipan_path.rsplit('/', maxsplit=1)
        alipan_parent_path, alipan_file_name = '/' + alipan_path_info_arr[0].removeprefix('/'), alipan_path_info_arr[1]
    else:
        alipan_parent_path, alipan_file_name = '/', alipan_path
    return baidupan_save_path_func(alipan_parent_path, alipan_file_name)


def init_file_md5(alipan_api: Aligo, alipan_file: BaseShareFile | BaseFile):
    if alipan_file.type == 'folder':
        return
    if not hasattr(alipan_file, 'md5') or not getattr(alipan_file, 'md5'):
        share_token = get_share_cache_token(alipan_api, alipan_file.share_id)
        save_result = alipan_api.share_file_saveto_drive(file_id=alipan_file.file_id, share_token=share_token)
        if not hasattr(save_result, 'message'):
            ali_pan_file = alipan_api.get_file(file_id=save_result.file_id)
            alipan_file.md5 = ali_pan_file.content_hash
            alipan_api.delete_file(file_id=save_result.file_id)


def alipan_file_save(baidupan_access_token: str, alipan_api: Aligo, alipan_path: str,
                     alipan_file: BaseShareFile | BaseFile,
                     transfer_list_file: str,
                     exist_file_list: list[str],
                     alipan_file_filter_func: Callable[[Aligo, str, BaseShareFile | BaseFile], bool] = lambda a, x,
                                                                                                              y: True,
                     baidupan_save_path_func: Callable[[str, str], str] =
                     lambda x, y: f'{x}/{y}',
                     baidupan_save_after_handle_func: Callable[[BaseFile, str, UploadCreateResult], None] = None,
                     is_init_file_md5: bool = True):
    """
    阿里云盘文件保存
    :param baidupan_access_token:百度网盘access_token
    :param alipan_api:
    :param alipan_path:
    :param alipan_file:
    :param transfer_list_file:
    :param exist_file_list:
    :param alipan_file_filter_func: 阿里云盘文件过滤函数，只有满足条件的文件才会进行转存，参数：(阿里云盘的路径(包含文件名): alipan_path, 阿里云盘的文件: alipan_file)-> True或False
    :param baidupan_save_path_func: 百度网盘路径函数：(阿里云盘的路径(不包含文件名，根路径为/): alipan_path, 阿里云盘的文件名: alipan_file_name)-> 百度网盘的路径
    :param baidupan_save_after_handle_func: 百度网盘保存后的处理函数，参数：(阿里云盘文件, 百度网盘保存路径（包含文件名）,上传保存结果)
    :param is_init_file_md5: 是否初始化文件的md5
    :return:
    """
    if alipan_file.file_id in exist_file_list:
        return
    if is_init_file_md5:
        init_file_md5(alipan_api=alipan_api, alipan_file=alipan_file)
    if not alipan_file_filter_func(alipan_api, alipan_path, alipan_file):
        return

    baidupan_save_path = get_baidupan_save_path(alipan_path, baidupan_save_path_func)
    try:
        old_file_info = baidu_pan_file_meta_tools.get_file_meta_by_path(baidupan_access_token, baidupan_save_path)
        if old_file_info and old_file_info.size == alipan_file.size:
            print(f'文件已经存在百度网盘：{old_file_info.path}')
            if baidupan_save_after_handle_func and isinstance(baidupan_save_after_handle_func, Callable):
                baidupan_save_after_handle_func(alipan_file, baidupan_save_path, old_file_info)
            return
        if isinstance(alipan_file, BaseShareFile):
            tmp_local_save_path = os.path.join(os_tools.get_user_home(),
                                               f'ali_tmp_file-{alipan_file.share_id}')
        else:
            tmp_local_save_path = os.path.join(os_tools.get_user_home(),
                                               f'ali_tmp_file-{hashlib_tools.calc_md5(transfer_list_file)}')
        os.makedirs(tmp_local_save_path, exist_ok=True)
        tmp_local_file = aliyun_pan_tools.download_file(alipan_api, pan_file=alipan_file,
                                                        local_path=tmp_local_save_path,
                                                        download_after_delete=isinstance(alipan_file, BaseShareFile))
        if tmp_local_file:
            result = baidu_pan_file_upload_tools.upload_file(baidupan_access_token,
                                                             local_file=tmp_local_file,
                                                             pan_path=baidupan_save_path)
            if result and result.errno == 0:
                if os.path.exists(tmp_local_file):
                    os.remove(tmp_local_file)
                with open(transfer_list_file, 'a+', encoding='utf-8') as f:
                    f.write(f'{alipan_file.file_id}\n')
            if baidupan_save_after_handle_func and isinstance(baidupan_save_after_handle_func, Callable):
                baidupan_save_after_handle_func(alipan_file, baidupan_save_path, result)
    except Exception as e:
        log_error(logger,

                  f'[alipan_share_to_baidupan]阿里云文件[{alipan_path}]转存到百度云[{baidupan_save_path}]出现异常', e)



"""
jieba: pip install jieba -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
import jieba


由于jieba分词速度慢，使用jieba_fast替换
pip3 install jieba_fast
"""
import jieba_fast as jieba


def word_cut(text_content: str, cut_all: bool = False, hmm: bool = True) -> list[str]:
    """
    精确模式分词
    @param text_content: 分词文本
    @param cut_all: 全模式分词, 从待分词内容的第一个字开始遍历，将每一个字作为词语的第一个字，返回所有可能的词语，会重复利用词语和字，因此也可能会出现多种含义。
    @param hmm: 根据HMM模型（隐马尔可夫模型）自动识别新词。
    @return: list[str]
    """
    cut_result = jieba.cut(text_content, cut_all=cut_all, HMM=hmm)
    return list(cut_result)


def word_cut_for_search(text_content: str) -> list[str]:
    """
    搜索引擎分词： 搜索引擎模式在精确模式的基础上，对精确模式中的长词，再按照全模式进一步分词，用于搜索时可以匹配到更多的结果。
    """
    cut_result = jieba.cut(text_content, cut_all=True)
    return list(cut_result)


if __name__ == '__main__':
    print(word_cut('迅雷不及掩耳盗铃儿响叮当仁不让世界充满爱之势'))
    print(word_cut_for_search('迅雷不及掩耳盗铃儿响叮当仁不让世界充满爱之势'))



from aligo import BaseShareFile


def is_folder(alipan_file: BaseShareFile) -> bool:
    """是否是文件夹"""
    return alipan_file.type == 'folder'


def is_file(alipan_file: BaseShareFile) -> bool:
    """是否是文件"""
    return not is_folder(alipan_file)


def is_audio(alipan_file: BaseShareFile) -> bool:
    """是否是音频"""
    return alipan_file.category == 'audio'


def is_video(alipan_file: BaseShareFile) -> bool:
    """是否是视频"""
    return alipan_file.category == 'video'


def is_doc(alipan_file: BaseShareFile) -> bool:
    """是否是文档"""
    return alipan_file.category == 'doc'


def is_image(alipan_file: BaseShareFile) -> bool:
    """是否是图片"""
    return alipan_file.category == 'image'





import os
from admin import admin_settings
from afeng_tools.baidu_pan_tool import baidu_pan_tools
from afeng_tools.baidu_pan_tool.tools.baidu_pan_file_path_tools import format_pan_path
from afeng_tools.baidu_pan_tool.transfer_tool.core.alipan_transfer_to_baidupan import alipan_file_save
from afeng_tools.file_tool import file_tools
from afeng_tools.os_tool import os_tools
from aligo import Aligo, BaseFile

from .. import aliyun_drive_auth_tools, aliyun_drive_file_tools, aliyun_drive_download_tools

baidupan_access_token = baidu_pan_tools.get_access_token()

transfer_list_file = os.path.join(os_tools.get_user_home(),
                                  f'.alipan_transfer_list-665dbcbcc3379ac1ab604c99b17d5683e0aceb3c')
exist_file_list = []
if os.path.exists(transfer_list_file):
    exist_file_list = file_tools.read_file_lines(transfer_list_file)


def handle_file(alipan_api: Aligo, pan_file: BaseFile):
    if pan_file.type == 'folder':
        return
    pan_file.download_url = aliyun_drive_download_tools.get_download_url(alipan_api, file_id=pan_file.file_id)
    alipan_file_save(
        baidupan_access_token=baidupan_access_token,
        alipan_api=alipan_api,
        alipan_path=pan_file.path,
        alipan_file=pan_file,
        transfer_list_file=transfer_list_file,
        exist_file_list=exist_file_list,
        alipan_file_filter_func=lambda a, x, y: True,
        baidupan_save_path_func=lambda x, y: f'/apps/阿里云转存/分享转存{format_pan_path(x)}/{format_pan_path(y)}',
        is_init_file_md5=False)


def run():
    alipan_api = aliyun_drive_auth_tools.get_alipan_api(use_resource_drive=True)
    file_id = '665dbcbcc3379ac1ab604c99b17d5683e0aceb3c'
    root_file = aliyun_drive_file_tools.get_file(alipan_api, file_id)
    aliyun_drive_file_tools.iterate_handle_all_file(alipan_api,
                                                    iterate_handle_func=handle_file,
                                                    parent_file_id=file_id, parent_path=f'/{root_file.name}')


if __name__ == '__main__':
    run()
    pass



import os
import re
from typing import Any

import requests

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.baidu_pan_tool import baidu_pan_tools
from afeng_tools.baidu_pan_tool.tools import baidu_pan_file_meta_tools
from afeng_tools.fastapi_tool.common.enum import ResourceUrlEnum, ResourceFormatEnum
from afeng_tools.fastapi_tool.common.po_service.resource_po_service_ import ResourcePoService
from afeng_tools.fastapi_tool.common.service.base_service import BaseService
from afeng_tools.http_tool import http_download_tools
from afeng_tools.sqlalchemy_tools.crdu import base_crdu


class ResourceService(BaseService):
    """
    使用示例：resource_service = ResourceService(app_info.db_code, ResourceInfoPo, app_code=app_info.code)
    """

    po_service_type = ResourcePoService

    def get_by_code(self, resource_code: str) -> Any:
        """
        通过编码查询
        :param resource_code: 资源编码
        :return: ResourceInfoPo
        """
        return self.po_service.get(self.po_model_type.resource_code == resource_code)

    def query_in_code(self, resource_code_list: list[str]) -> list[Any]:
        """通过编码列出"""
        return self.po_service.query_more(self.po_model_type.resource_code.in_(resource_code_list))

    @classmethod
    def get_resource_url(cls, url: str, resource_code: str) -> str:
        if url:
            return url
        if resource_code:
            return f'{ResourceUrlEnum.base_url.value}/{resource_code}'

    def refresh_baidu_img_access_url(self, resource_info_po) -> Any:
        """刷新百度图片资源的访问路径"""
        if resource_info_po and resource_info_po.resource_format == ResourceFormatEnum.image and resource_info_po.baidu_fs_id:
            baidu_access_token = baidu_pan_tools.get_access_token()
            file_info_list = baidu_pan_file_meta_tools.get_file_metas(baidu_access_token, [resource_info_po.baidu_fs_id], thumb=1)
            if file_info_list:
                access_url = file_info_list[0].thumbs.url3
                resource_info_po.access_url = access_url
                re_search = re.search('&time=(.*?)&', access_url)
                if re_search:
                    resource_info_po.expire_timestamp = int(re_search.group(1)) + 8 * 3600 - 100
                base_crdu.update(resource_info_po, db_code=self.db_code)
                return resource_info_po
        return resource_info_po

    @classmethod
    def get_baidu_download_url(cls, resource_info_po) -> str:
        """获取百度网盘文件下载链接"""
        if resource_info_po and resource_info_po.baidu_fs_id and resource_info_po.resource_format == ResourceFormatEnum.file:
            baidu_access_token = baidu_pan_tools.get_access_token()
            file_info_list = baidu_pan_file_meta_tools.get_file_metas(baidu_access_token, [resource_info_po.baidu_fs_id],
                                                            d_link=1)
            if file_info_list:
                file_info = file_info_list[0]
                # 官方不允许使用浏览器直接下载超过50MB的文件， 超过50MB的文件需用开发者原生的软件或者app进行下载
                # if file_info.size <= 50 * 1024 * 1024:
                down_url = file_info.dlink + f'&access_token={baidu_access_token}'
                down_resp = requests.head(down_url, headers={
                    'Host': 'd.pcs.baidu.com',
                    'User-Agent': 'pan.baidu.com'
                })
                if down_resp.status_code == 302:
                    return down_resp.headers.get('Location')

    @classmethod
    def run_local_cache(cls, resource_code: str, resource_url: str, subfix: str = None, local_cache_path: str = None):
        """运行本地缓存"""
        if local_cache_path is None:
            local_cache_path = os.path.join(settings_tools.get_config(SettingsKeyEnum.server_static_save_path), 'resource')
        os.makedirs(local_cache_path, exist_ok=True)
        if subfix:
            http_download_tools.download_file(resource_url, save_path=local_cache_path,
                                              save_file_name=f'{resource_code}{subfix}')
        else:
            http_download_tools.download_file(resource_url, save_path=local_cache_path, save_file_name=str(resource_code))

    def download_and_add_image_resource(self, image_title: str, image_url: str, group_path: str,
                                        image_name: str = None) -> Any:
        """下载并添加图片资源"""
        pan_path = f'/apps/www/{self.app_code}/image/{group_path}'
        include_filename = False
        if image_name:
            include_filename = True
            pan_path = pan_path + f'/{image_name}'
        result = baidu_pan_tools.save_to_pan(file_url=image_url,
                                             pan_path=pan_path,
                                             include_filename=include_filename)
        if result.errno == 0:
            return self.add_resource(result.fs_id, resource_format=ResourceFormatEnum.image.value,
                                     resource_name=f'[{image_title}]{result.server_filename}')

    def add_resource(self, fs_id: int, resource_format: ResourceFormatEnum, access_url: str = None,
                     download_flag: bool = False,
                     resource_name: str = None) -> Any:
        """添加资源"""
        po = self.po_model_type(
            resource_code=fs_id,
            baidu_fs_id=fs_id,
            resource_format=resource_format,
            download_flag=download_flag,
            resource_name=resource_name,
        )
        if access_url:
            po.access_url = access_url
            re_search = re.search('&time=(.*?)&', access_url)
            if re_search:
                po.expire_timestamp = int(re_search.group(1)) + 8 * 3600 - 100
        return base_crdu.save(po, self.po_model_type.resource_code == po.resource_code, exist_update=False,
                              db_code=self.db_code)



from afeng_tools.weixin_tool.core.model.item.wx_receive_event_models import WeixinSubscribeEventItem, WeixinQrEventItem, \
    WeixinScanEventItem, WeixinLocationEventItem, WeixinMenuEventItem, WeixinEventItem
from afeng_tools.weixin_tool.core.parse.receive_event.parse_tool import parse_event, parse_event_key, parse_ticket, \
    parse_location_x, parse_location_y, parse_location_precision
from afeng_tools.weixin_tool.core.parse.receive_msg.parse_tool import parse_to_user, parse_from_user, parse_create_time, \
    parse_msg_type


def parse_subscribe_event(openid, msg) -> WeixinSubscribeEventItem:
    """转换订阅/取消订阅事件表"""
    return WeixinSubscribeEventItem(
        openid=openid,
        to_user=parse_to_user(msg),
        from_user=parse_from_user(msg),
        create_time=parse_create_time(msg),
        msg_type=parse_msg_type(msg),
        event=parse_event(msg)
    )


def parse_qr_event(openid, msg) -> WeixinQrEventItem:
    """扫描带参数二维码事件"""
    return WeixinQrEventItem(
        openid=openid,
        to_user=parse_to_user(msg),
        from_user=parse_from_user(msg),
        create_time=parse_create_time(msg),
        msg_type=parse_msg_type(msg),
        event=parse_event(msg),
        event_key=parse_event_key(msg),
        ticket=parse_ticket(msg),
    )


def parse_scan_event(openid, msg) -> WeixinScanEventItem:
    """用户已关注时的事件推送"""
    return WeixinScanEventItem(
        openid=openid,
        to_user=parse_to_user(msg),
        from_user=parse_from_user(msg),
        create_time=parse_create_time(msg),
        msg_type=parse_msg_type(msg),
        event=parse_event(msg),
        event_key=parse_event_key(msg),
        ticket=parse_ticket(msg),
    )


def parse_location_event(openid, msg) -> WeixinLocationEventItem:
    """上报地理位置事件"""
    return WeixinLocationEventItem(
        openid=openid,
        to_user=parse_to_user(msg),
        from_user=parse_from_user(msg),
        create_time=parse_create_time(msg),
        msg_type=parse_msg_type(msg),
        event=parse_event(msg),
        location_x=parse_location_x(msg),
        location_y=parse_location_y(msg),
        precision=parse_location_precision(msg),
    )


def parse_menu_event(openid, msg) -> WeixinMenuEventItem:
    """点击自定义菜单事件"""
    return WeixinMenuEventItem(
        openid=openid,
        to_user=parse_to_user(msg),
        from_user=parse_from_user(msg),
        create_time=parse_create_time(msg),
        msg_type=parse_msg_type(msg),
        event=parse_event(msg),
        event_key=parse_event_key(msg),
    )


def parse(openid, msg) -> WeixinEventItem:
    """转换事件"""
    event = parse_event(msg)
    if event == 'unsubscribe':
        return parse_subscribe_event(openid, msg)
    if event == 'subscribe':
        event_key = parse_event_key(msg)
        if not event_key:
            return parse_subscribe_event(openid, msg)
        if event_key.startswith('qrscene_'):
            return parse_qr_event(openid, msg)
    if event == 'SCAN':
        return parse_scan_event(openid, msg)
    if event == 'LOCATION':
        return parse_location_event(openid, msg)
    if event == 'CLICK' or event == 'VIEW':
        return parse_menu_event(openid, msg)



import json

from playwright.sync_api import Page
from requests.cookies import RequestsCookieJar

from afeng_tools.serialization_tool import pickle_tools


def save_cookies(page: Page, auth_cookie_file: str):
    """保存Cookie"""
    cookie_list = page.context.cookies()
    cookie_str = json.dumps(cookie_list)
    with open(auth_cookie_file, 'wb') as cookie_file:
        cookie_file.write(cookie_str.encode('utf-8'))


def set_cookies(page: Page, auth_cookie_file: str):
    """设置Cookie"""
    with open(auth_cookie_file, 'rb') as cookie_file:
        cookie_str = cookie_file.read()
    cookie_list = json.loads(cookie_str)
    page.context.add_cookies(cookie_list)
    page.reload()


def get_cookies(auth_cookie_file: str) -> dict:
    """获取Cookie"""
    with open(auth_cookie_file, 'rb') as cookie_file:
        cookie_str = cookie_file.read()
    cookie_list = json.loads(cookie_str)
    cookie_dict = {}
    for tmp_cookie in cookie_list:
        cookie_dict[tmp_cookie['name']] = tmp_cookie['value']
    return cookie_dict


def save_cookie_jar(cookies: RequestsCookieJar, auth_cookie_file: str):
    """保存cookie"""
    pickle_tools.save_to_file([cookies], auth_cookie_file)


def get_cookie_jar(auth_cookie_file: str) -> RequestsCookieJar:
    """获取cookie"""
    return pickle_tools.parse_to_obj(auth_cookie_file)[0]


if __name__ == '__main__':
    cookies = RequestsCookieJar()
    cookies.set('LTWSESSID', '1ni1rd7u6agc468ogjubprtdda')
    cookies.set('SERVERID', '4cb8c33eaf77df5ec4c43a1d17a68293|1696528114|1696528114')
    cookies.set('cklogin', 'e58f53bd6e40770a3efdeb203825b879226e659ce9afdcec559f59f11d28dfada')
    cookies.set('mmopen', '2FpxGzl2aG2hnvCSoibR7cFralsdUpMG1nqViaSM9rF0HXoS50tGrRIMABlnvLJXialiaDwFsYeqcia6MpPyHhIicul7bg')
    save_cookie_jar(cookies, 'temp_cookie.txt')
    cookies = get_cookie_jar('temp_cookie.txt')
    print(cookies)


from typing import Optional

from starlette.requests import Request
from starlette.responses import RedirectResponse
from starlette.templating import _TemplateResponse
from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum
from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response_with_cache
from afeng_tools.fastapi_tool.fastapi_response_tools import resp_template
from afeng_tools.fastapi_tool.template.item import TemplateHtmlHeadData, TemplateShowQrcodeAreaData, \
    TemplateRedirectAreaData, AppinfoDataItem, TemplateAlertInfoAreaData
from afeng_tools.fastapi_tool.template.item.page_template_items import PageTemplateItem
from afeng_tools.fastapi_tool.template.service import template_render_service
from afeng_tools.http_tool import http_request_tools, http_params_tools
from afeng_tools.pydantic_tool.model.common_models import LinkItem


def render_template(request: Request, template_file: str, template_item: PageTemplateItem) -> _TemplateResponse:
    """响应模板"""
    return create_template_response_with_cache(request,
                                               template_file=template_file,
                                               context=template_item.model_dump())


def create_html_head_data(_app_info: AppInfo, is_mobile: bool, title: str, description: str = None,
                          keyword_list: list[str] = None,
                          author: str = None, favicon: str = None, origin: str = None,
                          custom_head_code_list: Optional[list[str]] = None) -> TemplateHtmlHeadData:
    """创建页面head"""
    if title is not None:
        title = f'{title}|{_app_info.title}|{_app_info.sub_title}'
    if description is None:
        description = f'{title}:{_app_info.description}'
    if keyword_list is None:
        keyword_list = [title]
        keyword_list.extend(_app_info.keywords)
    else:
        keyword_list = [tmp for tmp in keyword_list if tmp is not None]
    if author is None:
        author = 'chentiefeng'
    if favicon is None:
        favicon = '/favicon.ico'
    if origin is None:
        origin = _app_info.origin
    return TemplateHtmlHeadData(is_mobile=is_mobile, title=title, description=description,
                                keyword_list=keyword_list,
                                author=author, favicon=favicon, origin=origin, baidu_tm_id=_app_info.baidu_tm_id,
                                custom_head_code_list=custom_head_code_list)


def redirect_to(request: Request, target: str,
                app_info_item: AppinfoDataItem = AppinfoDataItem(title='阿锋之家'),
                page_title: str = '跳转中心',
                page_favicon_ico: str = '/static/image/favicon.ico',
                back_url: str = '/',
                ad_list: list[LinkItem] = None,
                template_file: str = 'common/views/redirect/index.html',
                group_code: str = 'common/redirect',
                is_redirect: bool = False,
                cache_html_file: str = None) -> _TemplateResponse:
    """重定向页面"""
    if not target:
        return RedirectResponse('/')
    if target.startswith('%2F'):
        target = http_params_tools.url_decode(target)
    is_mobile = http_request_tools.is_mobile(request.headers.get('user-agent'))
    context_data = {
        'is_mobile': is_mobile,
        'title': page_title,
        'favicon_ico': page_favicon_ico,
        'baidu_tm_id': app_info_item.baidu_tm_id,
    }
    _, area_template_file = template_render_service.get_template_redirect_area(
        group_code=group_code,
        context_data_func=lambda: TemplateRedirectAreaData(
            is_mobile=is_mobile,
            app_info=app_info_item,
            ad_list=ad_list,
            redirect_url=target,
            back_url=back_url
        ),
        reload_freq=ReloadFreqEnum.always)
    context_data['template_redirect_area'] = area_template_file
    return resp_template(request, template_file=template_file, context_data=context_data,
                         is_redirect=is_redirect, redirect_name='redirect',
                         cache_html_file=cache_html_file)


def redirect_to_qrcode_page(request: Request,
                            qrcode_image_url: str = '/static/image/qrcode/wx_of_qrcode.jpg',
                            qrcode_image_title: str = '公众号：阿锋之家',
                            message_list: list[str] = None,
                            page_title: str = '请关注公众号：阿锋之家',
                            page_favicon_ico: str = '/static/image/favicon.ico',
                            template_file: str = 'common/views/alert/index.html',
                            group_code: str = 'common/show_qrcode',
                            is_redirect: bool = False,
                            cache_html_file: str = None,
                            baidu_tm_id: str = None) -> _TemplateResponse:
    """重定向到扫码页"""
    is_mobile = http_request_tools.is_mobile(request.headers.get('user-agent'))
    context_data = {
        'is_mobile': is_mobile,
        'title': page_title,
        'favicon_ico': page_favicon_ico,
        'baidu_tm_id': baidu_tm_id,
    }
    if message_list is None:
        message_list = [
            '1、打开微信',
            '2、扫描上方二维码',
            '3、关注"阿锋之家"',
        ]
    _, area_template_file = template_render_service.get_template_show_qrcode_area(
        group_code=group_code,
        context_data_func=lambda: TemplateShowQrcodeAreaData(
            is_mobile=is_mobile,
            qrcode_image_url=qrcode_image_url,
            qrcode_image_title=qrcode_image_title,
            message_list=message_list
        ),
        reload_freq=ReloadFreqEnum.always)
    context_data['template_alert_area'] = area_template_file
    return resp_template(request, template_file=template_file, context_data=context_data,
                         is_redirect=is_redirect, redirect_name='qrcode',
                         cache_html_file=cache_html_file)


def redirect_to_alert_page(request: Request,
                           alert_title: str,
                           alert_message_list: list[str],
                           alert_logo_url: str = None,
                           page_title: str = None,
                           page_favicon_ico: str = '/static/image/favicon.ico',
                           template_file: str = 'common/views/alert/index.html',
                           group_code: str = 'common/alert/alert_info',
                           is_redirect: bool = False,
                           cache_html_file: str = None,
                           baidu_tm_id: str = None) -> _TemplateResponse:
    """重定向到警告页"""
    is_mobile = http_request_tools.is_mobile(request.headers.get('user-agent'))

    context_data = {
        'is_mobile': is_mobile,
        'title': page_title if page_title else alert_title,
        'favicon_ico': page_favicon_ico,
        'baidu_tm_id': baidu_tm_id,
    }
    _, area_template_file = template_render_service.get_template_alert_info_area(
        group_code=group_code,
        context_data_func=lambda: TemplateAlertInfoAreaData(
            is_mobile=is_mobile,
            alert_logo_url=alert_logo_url if alert_logo_url else TemplateAlertInfoAreaData.model_fields.get(
                'alert_logo_url').default,
            alert_title=alert_title,
            alert_message_list=alert_message_list
        ),
        reload_freq=ReloadFreqEnum.always)
    context_data['template_alert_area'] = area_template_file
    return resp_template(request, template_file=template_file, context_data=context_data,
                         is_redirect=is_redirect, redirect_name='alert',
                         cache_html_file=cache_html_file)



from typing import Any

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService
from afeng_tools.fastapi_tool.common.service import icon_base_service
from afeng_tools.pydantic_tool.model.common_models import LinkItem
from afeng_tools.sequence_tool import list_tools


class LinkPoService(PoService):
    """
    使用示例：link_po_service = LinkPoService(app_info.db_code, LinkInfoPo)
    """
    _table_name_ = "tb_link_info"

    def query_by_type_code(self, type_code: str) -> list[Any]:
        return self.query_more(self.model_type.type_code == type_code)

    def get_by_code(self, type_code: str, code: str) -> Any:
        return self.get(self.model_type.type_code == type_code, self.model_type.code == code)

    @classmethod
    def convert_to_link_item(cls, link_po, is_active: bool = False) -> LinkItem:
        return LinkItem(
            title=link_po.title,
            href=link_po.link_url,
            code=link_po.code,
            description=link_po.description,
            image=icon_base_service.get_icon_code(icon_type=link_po.icon_type,
                                                  icon_value=link_po.icon_value,
                                                  alt=link_po.title,
                                                  image_src=link_po.image_src),
            target=link_po.target.value if link_po.target else '_blank',
            is_ok=link_po.is_ok if link_po.is_ok is not None else True,
            is_active=is_active)

    @classmethod
    def convert_po_2_item(cls, data_list: list) -> list[LinkItem]:
        return [LinkItem(
            href=tmp.link_url,
            code=f'{tmp.type_code}__{tmp.code}',
            title=tmp.title,
            description=tmp.description,
            image=icon_base_service.get_icon_code(icon_type=tmp.icon_type,
                                                  icon_value=tmp.icon_value,
                                                  alt=tmp.title,
                                                  image_src=tmp.image_src),
            target=tmp.target.value if tmp.target else '_blank',
            is_ok=tmp.is_ok if tmp.is_ok is not None else True,
        ) for tmp in data_list] if data_list else []

    def query_link_tree(self, type_code: str, parent_code: str = None,
                        parent_code_dict: dict = None) -> list[LinkItem]:
        """查询链接树"""
        if parent_code_dict is None:
            all_link_list = self.query_more(self.model_type.type_code == type_code)
            parent_code_dict = list_tools.list_to_subgroup(group_by_attr='parent_code', data_list=all_link_list)
        result_list = []
        child_po_list = parent_code_dict.get(parent_code)
        if child_po_list:
            for tmp_child_po in child_po_list:
                tmp_link_item = self.convert_to_link_item(tmp_child_po)
                if parent_code_dict.get(tmp_child_po.code):
                    tmp_link_item.children = self.query_link_tree(type_code=type_code,
                                                                  parent_code=tmp_child_po.code,
                                                                  parent_code_dict=parent_code_dict)
                result_list.append(tmp_link_item)
        return result_list



from abc import ABCMeta, abstractmethod, ABC

from afeng_tools.weixin_tool.core import weixin_reply_tool
from afeng_tools.weixin_tool.core.model.item.wx_receive_event_models import WeixinEventItem, WeixinSubscribeEventItem
from afeng_tools.weixin_tool.core.model.item.wx_receive_msg_models import WeixinTextMsgItem, WeixinImageMsgItem, \
    WeixinVoiceMsgItem, WeixinVideoMsgItem, WeixinLocationMsgItem, WeixinLinkMsgItem
from afeng_tools.weixin_tool.core.response import XmlResponse


class WeixinTextMsgHandler(metaclass=ABCMeta):
    """微信文本消息处理"""

    def __init__(self):
        self.msg_handler = None
        self.msg_item = None

    def init(self, msg_handler: 'WeixinMsgHandler'):
        self.msg_handler = msg_handler
        self.msg_item = msg_handler.msg_item

    def handle(self) -> XmlResponse:
        return self.msg_handler.return_no_handle()


class WeixinMsgHandler(metaclass=ABCMeta):

    def __init__(self,
                 msg_model: WeixinTextMsgItem | WeixinImageMsgItem | WeixinVoiceMsgItem | WeixinVideoMsgItem | WeixinLocationMsgItem | WeixinLinkMsgItem | WeixinEventItem,
                 text_msg_handler: WeixinTextMsgHandler):
        self.msg_item = msg_model
        self.text_msg_handler = text_msg_handler
        self.text_msg_handler.init(self)

    def get_service_list(self) -> list[str]:
        """获取app服务列表"""
        return [f'<a href="{href}">{title}</a>' for title, href in self.get_all_service()]

    def handle(self) -> XmlResponse:
        # 查询是否在黑名单中
        if self.is_in_blacklist():
            return self.handle_in_blacklist()
        if isinstance(self.msg_item, WeixinTextMsgItem):
            return self.handle_text_msg()
        elif isinstance(self.msg_item, WeixinEventItem):
            if isinstance(self.msg_item, WeixinSubscribeEventItem):
                if getattr(self.msg_item, 'event') == 'subscribe':
                    return self.handle_subscribe()
                elif getattr(self.msg_item, 'event') == 'unsubscribe':
                    return self.handle_unsubscribe()
        return self.return_no_handle()

    def return_no_handle(self) -> XmlResponse:
        resp_msg_list = ['抱歉，暂无相关服务！', '现有如下服务：']
        resp_msg_list.extend(self.get_service_list())
        return weixin_reply_tool.reply_text(self.msg_item, '\n'.join(resp_msg_list))

    def handle_subscribe(self) -> XmlResponse:
        """关注的事件"""
        resp_msg_list = ['终于等到您了，为您提供了如下服务：']
        resp_msg_list.extend(self.get_service_list())
        resp_msg_list.append('（取消关注后，即使重新关注，也将无法使用服务！）')
        return weixin_reply_tool.reply_text(self.msg_item, '\n'.join(resp_msg_list))

    def handle_in_blacklist(self) -> XmlResponse:
        """处理黑名单中"""
        # 处理黑名单
        return weixin_reply_tool.reply_text(self.msg_item,
                                            '抱歉，你曾取消过关注，如果想要继续提供服务，请<a href="mailto:afenghome@aliyun.com" target="_blank">联系管理员</a>！')

    @abstractmethod
    def get_all_service(self) -> list[tuple[str, str]]:
        return []

    def handle_text_msg(self) -> XmlResponse:
        return self.text_msg_handler.handle()

    def is_in_blacklist(self) -> bool:
        """是否在黑名单中"""
        # black_list = blacklist_service.query_weixin_black_list()
        # # 查询用户黑名单
        # return self.msg_item.from_user in black_list
        return False

    def handle_unsubscribe(self) -> XmlResponse:
        """取消关注的事件"""
        # 将用户加入黑名单
        # black_po = BlacklistInfoPo(
        #     type_code='wx_blacklist',
        #     type_value=self.msg_item.from_user
        # )
        # # 将用户加入黑名单
        # blacklist_po_service.save(black_po,
        #                           BlacklistInfoPo.type_code == black_po.type_code,
        #                           BlacklistInfoPo.type_value == black_po.type_value)
        return weixin_reply_tool.reply_text(self.msg_item, 'success')


class DefaultWeixinMsgHandler(WeixinMsgHandler, ABC):

    def get_all_service(self) -> list[tuple[str, str]]:
        return [('阿锋书屋（QQ交流群：644437242）', 'https://www.afengbook.com')]




from typing import Any
from sqlalchemy import text
from sqlalchemy.orm import Session
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_class import SqlalchemyDb
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_items import DatabaseInfoItem
from afeng_tools.sqlalchemy_tools.crdu import sequence_crdu


class DBAdmin:
    """数据库管理"""
    def __init__(self, database_uri: str, echo_sql: bool = False):
        self.database_uri = database_uri
        self.sqlalchemy_db = SqlalchemyDb(DatabaseInfoItem(
            database_uri=database_uri,
            echo_sql=echo_sql
        ))

    def query_more_by_sql(self, sql: str) -> list[dict[str, Any]]:
        with self.sqlalchemy_db.engine.connect() as conn:
            cursor = conn.execute(text(sql))
            result_list = []
            for tmp_line in cursor.fetchall():
                result_list.append({tmp_key: tmp_line[index] for index, tmp_key in enumerate(cursor.keys())})
            return result_list

    def query_one_by_sql(self, sql: str) -> dict[str, Any]:
        with self.sqlalchemy_db.engine.connect() as conn:
            cursor = conn.execute(text(sql))
            sql_result = cursor.fetchone()
            return {tmp_key: sql_result[index] for index, tmp_key in enumerate(cursor.keys())}

    def query_by_sql(self, sql: str) -> list[tuple]:
        """通过sql语句查找"""
        with self.sqlalchemy_db.engine.connect() as conn:
            cursor = conn.execute(text(sql))
            return cursor.fetchall()

    def exec_by_sql(self, sql: str):
        """通过sql运行"""
        with self.sqlalchemy_db.engine.connect() as conn:
            cursor = conn.execute(text(sql))
            conn.commit()
            return cursor.lastrowid

    def get_session(self) -> Session:
        return self.sqlalchemy_db.get_session()

    def update_sequence_value_to(self, sequence_name: str, max_id: int):
        with self.sqlalchemy_db.get_session() as tmp_session:
            sequence_crdu.update_sequence_value_to(sequence_name, to_value=max_id, db=tmp_session)
            tmp_session.commit()

    def query_max_id(self, table_name: str):
        query_result = self.query_one_by_sql(f'SELECT max(id) FROM {table_name};')
        if query_result:
            return query_result.get('max')
        return 0

    def update_table_sequence_to_new(self, table_name: str):
        self.update_sequence_value_to(sequence_name=f'{table_name}_id_seq',
                                      max_id=self.query_max_id(table_name))


if __name__ == '__main__':
    db_admin = DBAdmin('postgresql://www_user:123456@127.0.0.1:5432/test_db')
    result = db_admin.query_one_by_sql('SELECT max(id) FROM tb_blacklist_info;')
    # result = db_admin.exec_by_sql('SELECT * FROM tb_blacklist_info;')
    print()



from setuptools import setup

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='thekey',
    version='2.0.0',
    description='Help You Find Any Text Queqly!|帮你快速查询文本！',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='xxxx',
    author_email='python@xxxx.com',
    py_modules=['thekey'],
    install_requires=[],
    entry_points={
        'console_scripts': [
            'findit=thekey:findit']
    }
)


from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class WeixinEventItem(BaseModel):
    """微信事件基本信息"""
    id: Optional[int] = None
    # OpenID
    openid: Optional[str] = None
    # 开发者微信号
    to_user: Optional[str] = None
    # 发送方帐号（一个OpenID）
    from_user: Optional[str] = None
    # 消息创建时间 （整型）
    create_time: Optional[int] = None
    # 消息类型，event
    msg_type: Optional[str] = None
    # 添加时间
    add_time: Optional[datetime] = None


class WeixinSubscribeEventItem(WeixinEventItem):
    """关注/取消关注事件表"""
    # 事件类型，subscribe(订阅)、unsubscribe(取消订阅)
    event: Optional[str] = None


class WeixinQrEventItem(WeixinEventItem):
    """扫描带参数二维码事件"""
    # 事件类型，subscribe
    event: Optional[str] = None
    # 事件 KEY 值，qrscene_为前缀，后面为二维码的参数值
    event_key: Optional[str] = None
    # 二维码的ticket，可用来换取二维码图片
    ticket: Optional[str] = None


class WeixinScanEventItem(WeixinEventItem):
    """用户已关注时的事件推送"""
    # 事件类型，SCAN
    event: Optional[str] = None
    # 事件 KEY 值，是一个32位无符号整数，即创建二维码时的二维码scene_id
    event_key: Optional[str] = None
    # 二维码的ticket，可用来换取二维码图片
    ticket: Optional[str] = None

    def __repr__(self):
        return self.event_key


class WeixinLocationEventItem(WeixinEventItem):
    """上报地理位置事件"""
    # 事件类型，LOCATION
    event: Optional[str] = None
    # 地理位置纬度
    location_x: Optional[str] = None
    # 地理位置经度
    location_y: Optional[str] = None
    # 地理位置精度
    precision: Optional[str] = None

    def __repr__(self):
        return self.location_x, self.location_y


class WeixinMenuEventItem(WeixinEventItem):
    """点击自定义菜单事件"""
    # 事件类型，CLICK:点击菜单拉取消息, VIEW:点击菜单跳转链接时的事件推送
    event: Optional[str] = None
    # 事件 KEY 值，与自定义菜单接口中KEY值对应 或 设置的跳转URL
    event_key: Optional[str] = None




"""
loguru工具类： pip install loguru  -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import traceback

from loguru import logger

from afeng_tools.log_tool import logger_settings
from afeng_tools.log_tool.logger_enums import LoggerConfigKeyEnum


def get_logger():
    info_file = logger_settings.get_config(LoggerConfigKeyEnum.info_file)
    info_rotation = logger_settings.get_config(LoggerConfigKeyEnum.info_rotation)
    if not info_rotation:
        info_rotation = '50 MB'
    if info_file:
        logger.add(info_file, rotation=info_rotation, compression='zip', encoding='utf-8', level='INFO')
    error_file = logger_settings.get_config(LoggerConfigKeyEnum.error_file)
    error_rotation = logger_settings.get_config(LoggerConfigKeyEnum.error_rotation)
    if not error_rotation:
        error_rotation = '30 MB'
    if error_file:
        logger.add(error_file, rotation=error_rotation, compression='zip', encoding='utf-8', level='WARNING')
    return logger


def log_error(log, error_msg: str, ex: Exception = None):
    """记录错误日志"""
    if ex:
        log.error(f'{error_msg}:{ex}\n {traceback.format_exc()}')
    else:
        log.error(f'{error_msg}\n {traceback.format_exc()}')



from typing import Any

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService
from afeng_tools.fastapi_tool.common.service import icon_base_service
from afeng_tools.pydantic_tool.model.common_models import LinkItem


class ArticlePoService(PoService):
    """
    使用示例：article_po_service = ArticlePoService(app_info.db_code, ArticleInfoPo)
    """
    _table_name_ = "tb_article_info"

    def get_by_code(self, article_code: str, type_code: str = None) -> Any:
        if type_code:
            return self.get(self.model_type.type_code == type_code, self.model_type.code == article_code)
        return self.get(self.model_type.code == article_code)

    @classmethod
    def convert_to_link_item(cls, article_po, is_active: bool = False,
                             article_href_prefix: str = '/article') -> LinkItem:
        image_code = None
        if hasattr(article_po, 'icon_type') and hasattr(article_po, 'icon_value'):
            image_code = icon_base_service.get_icon_code(icon_type=article_po.icon_type,
                                                         icon_value=article_po.icon_value,
                                                         alt=article_po.title,
                                                         image_src=article_po.image_src)
        return LinkItem(
            target='_blank',
            title=article_po.title,
            href=f'{article_href_prefix}/{article_po.type_code}/{article_po.code}',
            code=article_po.code,
            description=article_po.description,
            image=image_code,
            is_active=is_active)


class ArticleDetailPoService(PoService):
    """
    使用示例：article_detail_po_service = ArticleDetailPoService(app_info.db_code, ArticleDetailInfoPo)
    """
    _table_name_ = "tb_article_detail_info"

    def get_by_id(self, article_id: int) -> Any:
        return self.get(self.model_type.article_id == article_id)




"""
pickle序列化与反序列化工具
"""
import pickle
from typing import Any


def save_list_to_file(obj_list: list[object], save_file: str):
    """保存列表到文件"""
    with open(save_file, "wb") as f:
        # 序列化
        pickle.dump(len(obj_list), f)
        for tmp_obj in obj_list:
            # 序列化
            pickle.dump(tmp_obj, f)


def save_to_file(obj: object, save_file: str):
    """保存到文件"""
    if isinstance(obj, list):
        save_list_to_file(obj, save_file)
    else:
        with open(save_file, "wb") as f:
            # 序列化
            pickle.dump(obj, f)


def parse_to_obj_list(save_file: str) -> list[Any]:
    """保存到文件"""
    with open(save_file, "rb") as f:
        list_size = pickle.load(f)
        # 反序列
        return [pickle.load(f) for tmp in range(list_size)]


def parse_to_obj(save_file: str, is_list: bool = True) -> Any | list[Any]:
    """保存到文件"""
    if is_list:
        return parse_to_obj_list(save_file)
    else:
        with open(save_file, "rb") as f:
            # 反序列
            return pickle.load(f)



"""
http的mimetypes工具
"""
import mimetypes


def get_mimetype(file_name) -> tuple[str, str]:
    """
    通过名称获取 mimetype
    :param file_name: 文件名
    :return: 与文件名匹配的 mimetype,  tuple(type, encoding)
    """
    return mimetypes.guess_type(file_name)



from typing import Any

from afeng_tools.fastapi_tool.common.po_service.category_po_service_ import CategoryPoService
from afeng_tools.fastapi_tool.common.service.base_service import BaseService
from afeng_tools.pydantic_tool.model.common_models import LinkItem


class CategoryService(BaseService):
    """
    使用示例：category_service = CategoryService(app_info.db_code, CategoryInfoPo)
    """

    po_service_type = CategoryPoService

    def _up_tree_category(self, category_code: int, category_dict: dict[int, Any],
                          recursion: bool = True,
                          include_root: bool = True,
                          is_root: bool = True) -> list[Any]:
        """向上获取递归获取分类信息"""
        category_list = []
        tmp_category = category_dict.get(category_code)
        if tmp_category:
            if include_root:
                category_list.append(tmp_category)
            if tmp_category.parent_code:
                parent_category = category_dict.get(tmp_category.parent_code)
                if parent_category:
                    category_list.append(parent_category)
                    if recursion:
                        category_list.extend(
                            self._up_tree_category(parent_category.parent_code, category_dict, recursion=recursion,
                                                   include_root=True, is_root=False))
        if is_root:
            category_list.reverse()
        return category_list

    def _down_tree_category(self, category_code: int, category_dict: dict[int, Any],
                            recursion: bool = True,
                            include_root: bool = True) -> list[Any]:
        """
        向下获取递归获取分类信息， 第一项是当前分类
        :param category_code:
        :param category_dict:
        :param recursion: 是否层级递归
        :return:
        """
        category_list = []
        tmp_category = category_dict.get(category_code)
        if tmp_category:
            if include_root:
                category_list.append(tmp_category)
            for tmp_po in sorted(category_dict.values(), key=lambda x: x.order_num, reverse=True):
                if tmp_po.parent_code == category_code:
                    category_list.append(tmp_po)
                    if recursion:
                        category_list.extend(self._down_tree_category(tmp_po.code, category_dict,
                                                                      recursion=recursion,
                                                                      include_root=False))
        return category_list

    def get_category_list(self, category_code: int, group_code: int = None, category_dict: dict[int, Any] = None,
                          recursion: bool = True,
                          up: bool = True,
                          include_root: bool = True) -> list[Any]:
        """
        通过分类编码获取分类列表: 如：['程序开发','Java','Java虚拟机']
        :param category_code:
        :param group_code: 分组编码
        :param category_dict:
        :param recursion: 是否递归，True: 递归获取； False: 只获取父级或子集
        :param up:
            当为True时，向上获取，如当前是Java虚拟机,获取到的是['程序开发','Java','Java虚拟机']
            当为False时，向下获取，如当前是Java,获取到的是['Java基础','Java安全','Java虚拟机', ,'JavaWeb开发']
        :param include_root:
        :return: 分类列表
        """
        if category_dict is None:
            if group_code is None:
                category_info = self.po_service.get_one(self.po_model_type.code == category_code)
                group_code = category_info.group_code
            category_dict = {tmp.code: tmp for tmp in self.po_service.query_group_data(group_code)}
        return self._up_tree_category(category_code, category_dict,
                                      recursion=recursion,
                                      include_root=include_root) if up else self._down_tree_category(category_code,
                                                                                                     category_dict,
                                                                                                     recursion=recursion,
                                                                                                     include_root=include_root)

    def get_up_category_title_list(self, category_code: int, group_code: int = None) -> list[str]:
        """获取向上的分类标题"""
        category_list = self.get_category_list(group_code=group_code, category_code=category_code,
                                               recursion=True, up=True)
        return [tmp.title for tmp in category_list]

    def get_category_link_list(self, group_code: str, parent_code: str = None,
                               category_dict: dict[str, Any] = None,
                               is_recursion: bool = False) -> list[LinkItem]:
        """获取分类链接列表"""
        result_list = []
        if category_dict is None:
            category_dict = {tmp.code: tmp for tmp in self.po_service.query_group_data(group_code) if tmp.is_enable}
        for tmp_category in sorted([tmp for tmp in category_dict.values() if tmp.parent_code == parent_code],
                                   key=lambda x: x.order_num, reverse=True):
            tmp_link_item = self.po_service.convert_to_link_item(tmp_category)
            if is_recursion:
                tmp_link_item.children = self.get_category_link_list(group_code=group_code,
                                                                     parent_code=tmp_category.code,
                                                                     category_dict=category_dict,
                                                                     is_recursion=is_recursion)
            result_list.append(tmp_link_item)
        return result_list



from typing import List

from afeng_tools.aliyun_drive_tool.core.custom_aligo import CustomAligo
from aligo import Aligo, CreateFileResponse, MoveFileToTrashResponse, BatchSubResponse, MoveFileResponse, \
    CopyFileResponse, BaseFile
from aligo.types.Enum import CheckNameMode


def create_folder(alipan_api: Aligo, name: str,
                  parent_file_id: str = 'root', drive_id: str = None,
                  check_name_mode: CheckNameMode = 'auto_rename') -> CreateFileResponse:
    """
    创建文件夹
    :param alipan_api: Aligo
    :param name: [str] 文件夹名
    :param parent_file_id: Optional[str] 父文件夹id, 默认为 'root'
    :param drive_id: Optional[str] 指定网盘id, 默认为 None
    :param check_name_mode: Optional[CheckNameMode] 检查文件名模式, 默认为 'auto_rename'
    :return: [CreateFileResponse]
    """
    return alipan_api.create_folder(name=name,
                                    parent_file_id=parent_file_id,
                                    drive_id=drive_id,
                                    check_name_mode=check_name_mode)


def rename_file(alipan_api: Aligo, file_id: str, new_name: str,
                check_name_mode: CheckNameMode = 'refuse',
                drive_id: str = None) -> BaseFile:
    """重命名文件"""
    return alipan_api.rename_file(file_id=file_id, name=new_name, check_name_mode=check_name_mode, drive_id=drive_id)


def batch_rename_files(alipan_api: Aligo, file_id_list: List[str], new_name_list: List[str],
                       check_name_mode: CheckNameMode = 'refuse',
                       drive_id: str = None) -> List[BaseFile]:
    return alipan_api.batch_rename_files(file_id_list=file_id_list, new_name_list=new_name_list,
                                         check_name_mode=check_name_mode, drive_id=drive_id)


def copy_file(alipan_api: Aligo, file_id: str, to_parent_file_id: str = 'root', new_name: str = None,
              drive_id: str = None,
              to_drive_id: str = None) -> CopyFileResponse:
    """复制文件"""
    return alipan_api.copy_file(file_id=file_id, to_parent_file_id=to_parent_file_id, new_name=new_name,
                                drive_id=drive_id, to_drive_id=to_drive_id)


def batch_copy_file(alipan_api: Aligo, file_id_list: List[str], to_parent_file_id: str = 'root',
                    drive_id: str = None) -> List[BatchSubResponse[CopyFileResponse]]:
    """复制文件"""
    return alipan_api.batch_move_files(file_id_list=file_id_list, to_parent_file_id=to_parent_file_id,
                                       drive_id=drive_id)


def move_file(alipan_api: Aligo, file_id: str, to_parent_file_id: str = 'root', new_name: str = None,
              drive_id: str = None,
              to_drive_id: str = None) -> MoveFileResponse:
    """移动文件"""
    return alipan_api.move_file(file_id=file_id, to_parent_file_id=to_parent_file_id, new_name=new_name,
                                drive_id=drive_id, to_drive_id=to_drive_id)


def batch_move_file(alipan_api: Aligo, file_id_list: List[str], to_parent_file_id: str = 'root',
                    drive_id: str = None) -> List[BatchSubResponse[MoveFileResponse]]:
    """批量移动文件"""
    return alipan_api.batch_move_files(file_id_list=file_id_list, to_parent_file_id=to_parent_file_id,
                                       drive_id=drive_id)


def delete_file(alipan_api: CustomAligo, file_id: str, to_trash: bool = True,
                drive_id: str = None) -> MoveFileToTrashResponse | bool:
    """删除文件"""
    if to_trash:
        return alipan_api.move_file_to_trash(file_id=file_id, drive_id=drive_id)
    else:
        return alipan_api.delete_file(file_id=file_id, drive_id=drive_id)


def batch_delete_file(alipan_api: CustomAligo, file_id_list: List[str], to_trash: bool = True,
                      drive_id: str = None) -> List[BatchSubResponse]:
    """删除文件"""
    if to_trash:
        return alipan_api.batch_move_to_trash(file_id_list=file_id_list, drive_id=drive_id)
    else:
        return alipan_api.batch_delete_files(file_id_list=file_id_list, drive_id=drive_id)


def clear_recyclebin(alipan_api: CustomAligo, drive_id: str = None) -> bool:
    """清空回收站"""
    return alipan_api.clear_recyclebin(drive_id=drive_id)


if __name__ == '__main__':
    pass



import os

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.application_models import AppInfo


def get_html_cache_path(app_info: AppInfo = None) -> str:
    """获取html缓存路径"""
    if app_info is None:
        return os.path.join(settings_tools.get_config('server.html_save_path'), 'cache')
    return os.path.join(settings_tools.get_config('server.html_save_path'), 'cache', app_info.code)




def tree_get_value(data_dict: dict, key_list: list[str]):
    """
    递归获取字典的值
    :param data_dict:数据字典，如:  {'app' :{'log': {'error': {'file': 'test.log'}}}}
    :param key_list: 层级的键列表，如：['app', 'log', 'error', 'file']
    :return: 获取到的值, 如：'test.log'
    """
    if data_dict is None:
        return None
    if len(key_list) > 1:
        return tree_get_value(data_dict.get(key_list[0]), key_list[1:])
    else:
        return data_dict.get(key_list[0])


def tree_set_value(data_dict: dict, key_list: list[str], value):
    """
    递归设置字典的值
    :param data_dict: 数据子弹
    :param key_list: 键值列表， [app, static, path]
    :param value: 值
    :return:
    """
    if not data_dict.get(key_list[0]):
        data_dict[key_list[0]] = dict()
    if len(key_list) > 1:
        tree_set_value(data_dict[key_list[0]], key_list[1:], value)
    else:
        data_dict[key_list[0]] = value


def format_config_dict(config_dict: dict):
    """
    格式化配置字典
    :param config_dict:
        如：{
            'app.port': 18080,
            'app.log.info.file': 'log_error.log',
            'app.log.error.file': 'log_error.log',
            'app.static.url': '/static',
            'app.static.path': '/data/www/static',
        }
    :return: 格式化后的字典
            {
                "app": {
                    "port": 18080,
                    "log": {
                        "info": {
                            "file": "log_error.log"
                        },
                        "error": {
                            "file": "log_error.log"
                        }
                    },
                    "static": {
                        "url": "/static",
                        "path": "/data/www/static"
                    }
                },
                "app.port": 18080,
                "app.log.info.file": "log_error.log",
                "app.log.error.file": "log_error.log",
                "app.static.url": "/static",
                "app.static.path": "/data/www/static"
            }
    """
    result_data_dict = dict()
    for tmp_key, tmp_value in config_dict.items():
        if '.' in tmp_key:
            tmp_key_list = tmp_key.split('.')
            tree_set_value(result_data_dict, tmp_key_list, tmp_value)
        result_data_dict[tmp_key] = tmp_value
    return result_data_dict




from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class TmpSortPoService(PoService):
    """
    使用示例：tmp_sort_po_service = TmpSortPoService(app_info.db_code, TmpSortInfoPo)
    """
    _table_name_ = "tmp_sort_info"

    def delete_by_unique_code(self, unique_code: str):
        """通过唯一码删除"""
        return self.delete(self.model_type.unique_code == unique_code)

    def add_in_list(self, unique_code: str, in_data_list: list):
        """添加in列表"""
        tmp_po_list = []
        for index, tmp_in_data in enumerate(in_data_list):
            tmp_po_list.append(self.model_type(
                type_code='default',
                unique_code=unique_code,
                sort_value=str(tmp_in_data),
                sort_index=(index + 1)
            ))
        return self.add_batch(tmp_po_list)



from enum import Enum


class MobileKeycode(Enum):
    """安卓按键code值"""
    KEYCODE_UNKNOWN = 0
    # 左键
    KEYCODE_SOFT_LEFT = 1
    # 右键
    KEYCODE_SOFT_RIGHT = 2
    # home键
    KEYCODE_HOME = 3
    # back键
    KEYCODE_BACK = 4
    KEYCODE_CALL = 5
    KEYCODE_ENDCALL = 6
    KEYCODE_0 = 7
    KEYCODE_1 = 8
    KEYCODE_2 = 9
    KEYCODE_3 = 10
    KEYCODE_4 = 11
    KEYCODE_5 = 12
    KEYCODE_6 = 13
    KEYCODE_7 = 14
    KEYCODE_8 = 15
    KEYCODE_9 = 16
    KEYCODE_STAR = 17
    KEYCODE_POUND = 18
    KEYCODE_DPAD_UP = 19
    KEYCODE_DPAD_DOWN = 20
    KEYCODE_DPAD_LEFT = 21
    KEYCODE_DPAD_RIGHT = 22
    KEYCODE_DPAD_CENTER = 23
    # 声音放大
    KEYCODE_VOLUME_UP = 24
    # 声音缩小
    KEYCODE_VOLUME_DOWN = 25
    # 电源键
    KEYCODE_POWER = 26
    KEYCODE_CAMERA = 27
    KEYCODE_CLEAR = 28
    KEYCODE_A = 29
    KEYCODE_B = 30
    KEYCODE_C = 31
    KEYCODE_D = 32
    KEYCODE_E = 33
    KEYCODE_F = 34
    KEYCODE_G = 35
    KEYCODE_H = 36
    KEYCODE_I = 37
    KEYCODE_J = 38
    KEYCODE_K = 39
    KEYCODE_L = 40
    KEYCODE_M = 41
    KEYCODE_N = 42
    KEYCODE_O = 43
    KEYCODE_P = 44
    KEYCODE_Q = 45
    KEYCODE_R = 46
    KEYCODE_S = 47
    KEYCODE_T = 48
    KEYCODE_U = 49
    KEYCODE_V = 50
    KEYCODE_W = 51
    KEYCODE_X = 52
    KEYCODE_Y = 53
    KEYCODE_Z = 54
    KEYCODE_COMMA = 55
    KEYCODE_PERIOD = 56
    KEYCODE_ALT_LEFT = 57
    KEYCODE_ALT_RIGHT = 58
    KEYCODE_SHIFT_LEFT = 59
    KEYCODE_SHIFT_RIGHT = 60
    KEYCODE_TAB = 61
    KEYCODE_SPACE = 62
    KEYCODE_SYM = 63
    KEYCODE_EXPLORER = 64
    KEYCODE_ENVELOPE = 65
    KEYCODE_ENTER = 66
    KEYCODE_DEL = 67
    KEYCODE_GRAVE = 68
    KEYCODE_MINUS = 69
    KEYCODE_EQUALS = 70
    KEYCODE_LEFT_BRACKET = 71
    KEYCODE_RIGHT_BRACKET = 72
    KEYCODE_BACKSLASH = 73
    KEYCODE_SEMICOLON = 74
    KEYCODE_APOSTROPHE = 75
    KEYCODE_SLASH = 76
    KEYCODE_AT = 77
    KEYCODE_NUM = 78
    KEYCODE_HEADSETHOOK = 79
    # Camera focus
    KEYCODE_FOCUS = 80
    KEYCODE_PLUS = 81
    # 菜单键
    KEYCODE_MENU = 82
    KEYCODE_NOTIFICATION = 83
    KEYCODE_SEARCH = 84
    KEYCODE_MEDIA_PLAY_PAUSE = 85
    KEYCODE_MEDIA_STOP = 86
    KEYCODE_MEDIA_NEXT = 87
    KEYCODE_MEDIA_PREVIOUS = 88
    KEYCODE_MEDIA_REWIND = 89
    KEYCODE_MEDIA_FAST_FORWARD = 90
    KEYCODE_MUTE = 91


from sqlalchemy import BigInteger, Column, Text, String, Enum, Boolean, DateTime, func, Float, Integer

from afeng_tools.fastapi_tool.common.enum import IconTypeEnum, ResourceFormatEnum, SitemapFreqEnum, LinkTargetEnum
from afeng_tools.sqlalchemy_tools.core import sqlalchemy_base_model


class CountMixin:
    """数量Mixin"""
    _count: int = 0

    @property
    def count(self):
        return self._count

    @count.setter
    def count(self, count_value):
        self._count = count_value


class CommonPo:
    """
    通用模型: 使用示例：
        common_po = CommonPo(db_code=app_info.db_code)
        GroupInfoPo, CategoryInfoPo, ResourceInfoPo, LinkInfoPo, TagInfoPo, BlacklistInfoPo, SitemapInfoPo, ArticleInfoPo,ArticleDetailInfoPo = (common_po.GroupInfoPo, common_po.CategoryInfoPo, common_po.ResourceInfoPo, common_po.LinkInfoPo, common_po.TagInfoPo,common_po.BlacklistInfoPo, common_po.SitemapInfoPo,
                                            common_po.ArticleInfoPo, common_po.ArticleDetailInfoPo)
    """

    def __init__(self, db_code: str):
        self.db_code = db_code
        self.BaseModel = sqlalchemy_base_model.get_base_model(db_code=db_code)
        self._GroupInfoPo = None
        self._CategoryInfoPo = None
        self._ResourceInfoPo = None
        self._LinkInfoPo = None
        self._TagInfoPo = None
        self._TagRelationInfoPo = None
        self._BlacklistInfoPo = None
        self._SitemapInfoPo = None
        self._ArticleInfoPo = None
        self._ArticleDetailInfoPo = None
        self._CountInfoPo = None
        self._TmpSortInfoPo = None
        self._RelationInfoPo = None

    @property
    def GroupInfoPo(self):
        if self._GroupInfoPo is not None:
            return self._GroupInfoPo

        class GroupInfoPo(self.BaseModel, CountMixin):
            """分组信息"""
            __tablename__ = "tb_group_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            description = Column(Text, comment='描述', unique=False, nullable=True)
            image_src = Column(Text, comment='图标源地址', unique=False, nullable=True)
            icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]),
                               comment='图标类型',
                               unique=False, nullable=True)
            icon_value = Column(Text, comment='图标值', unique=False, nullable=True)

        self._GroupInfoPo = GroupInfoPo
        return self._GroupInfoPo

    @property
    def CategoryInfoPo(self):
        if self._CategoryInfoPo is not None:
            return self._CategoryInfoPo

        class CategoryInfoPo(self.BaseModel, CountMixin):
            """分类信息"""
            __tablename__ = "tb_category_info"
            group_code = Column(String(100), comment='分组编码', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            description = Column(Text, comment='描述', unique=False, nullable=True)
            image_src = Column(Text, comment='图标源地址', unique=False, nullable=True)
            icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]),
                               comment='图标类型', unique=False, nullable=True)
            icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
            parent_code = Column(String(100), comment='父编码', unique=False, nullable=True)

        self._CategoryInfoPo = CategoryInfoPo
        return self._CategoryInfoPo

    @property
    def ResourceInfoPo(self):
        if self._ResourceInfoPo is not None:
            return self._ResourceInfoPo

        class ResourceInfoPo(self.BaseModel):
            """资源信息"""
            __tablename__ = "tb_resource_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            resource_code = Column(String(100), comment='资源编码', index=True, unique=True, nullable=True)
            resource_name = Column(String(255), comment='资源名称', unique=False, nullable=True)
            resource_format = Column(Enum(ResourceFormatEnum), comment='资源格式（image:图片）', unique=False,
                                     nullable=True)
            local_path = Column(String(500), comment='本地路径', unique=False, nullable=True)
            baidu_fs_id = Column(BigInteger, comment='百度fs_id', unique=False, nullable=True)
            baidu_md5 = Column(String(255), comment='文件在百度网盘的md5', unique=False, nullable=True)
            baidu_size = Column(BigInteger, comment='在百度网盘的文件大小', unique=False, nullable=True)
            access_url = Column(String(500), comment='访问url', unique=False, nullable=True)
            expire_timestamp = Column(BigInteger, comment='过期时间戳', unique=False, nullable=True)
            download_flag = Column(Boolean, comment='下载标志', default=False)

        self._ResourceInfoPo = ResourceInfoPo
        return self._ResourceInfoPo

    @property
    def LinkInfoPo(self):
        if self._LinkInfoPo is not None:
            return self._LinkInfoPo

        class LinkInfoPo(self.BaseModel):
            """链接信息"""
            __tablename__ = "tb_link_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(100), comment='标题', unique=False, nullable=True)
            description = Column(Text, comment='描述', unique=False, nullable=True)
            image_src = Column(Text, comment='图标源地址', unique=False, nullable=True)
            icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]),
                               comment='图标类型', unique=False, nullable=True)
            icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
            link_url = Column(String(500), comment='链接地址', unique=False, nullable=True)
            parent_code = Column(String(100), comment='父编码', unique=False, nullable=True)
            target = Column(Enum(LinkTargetEnum), comment='打开方式', default=LinkTargetEnum.blank, unique=False,
                            nullable=True)
            is_ok = Column(Boolean, comment='是否正常访问', default=True)

        self._LinkInfoPo = LinkInfoPo
        return self._LinkInfoPo

    @property
    def TagInfoPo(self):
        if self._TagInfoPo is not None:
            return self._TagInfoPo

        class TagInfoPo(self.BaseModel, CountMixin):
            """标签信息"""
            __tablename__ = "tb_tag_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(100), comment='标题', unique=False, nullable=True)
            description = Column(String(255), comment='描述', unique=False, nullable=True)

        self._TagInfoPo = TagInfoPo
        return self._TagInfoPo

    @property
    def TagRelationInfoPo(self):
        if self._TagRelationInfoPo is not None:
            return self._TagRelationInfoPo

        class TagRelationInfoPo(self.BaseModel):
            """标签关联"""
            __tablename__ = "tb_tag_relation_info"
            tag_code = Column(String(100), comment='标签编码', unique=False, nullable=False)
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            type_value = Column(String(255), comment='类型值', unique=False, nullable=True)

        self._TagRelationInfoPo = TagRelationInfoPo
        return self._TagRelationInfoPo

    @property
    def ArticleInfoPo(self):
        if self._ArticleInfoPo is not None:
            return self._ArticleInfoPo

        class ArticleInfoPo(self.BaseModel):
            """帮助信息"""
            __tablename__ = "tb_article_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=False, nullable=True, index=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            sub_title = Column(String(255), comment='副标题', unique=False, nullable=True)
            description = Column(String(336), comment='描述', unique=False, nullable=True)
            publish_time = Column(DateTime, comment='发布时间', default=func.now())

        self._ArticleInfoPo = ArticleInfoPo
        return self._ArticleInfoPo

    @property
    def ArticleDetailInfoPo(self):
        if self._ArticleDetailInfoPo is not None:
            return self._ArticleDetailInfoPo

        class ArticleDetailInfoPo(self.BaseModel):
            """帮助信息"""
            __tablename__ = "tb_article_detail_info"
            article_id = Column(BigInteger, comment='文章主键', index=True, unique=True, nullable=False)
            content = Column(Text, comment='内容', unique=False, nullable=True)

        self._ArticleDetailInfoPo = ArticleDetailInfoPo
        return self._ArticleDetailInfoPo

    @property
    def CountInfoPo(self):
        if self._CountInfoPo is not None:
            return self._CountInfoPo

        class CountInfoPo(self.BaseModel):
            """数量信息"""
            __tablename__ = "tb_count_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            type_value = Column(String(255), comment='类型值', unique=False, nullable=True)
            count_value = Column(BigInteger, comment='数量值', default=0, unique=False)

        self._CountInfoPo = CountInfoPo
        return self._CountInfoPo

    @property
    def RelationInfoPo(self):
        if self._RelationInfoPo is not None:
            return self._RelationInfoPo

        class RelationInfoPo(self.BaseModel):
            """关联信息"""
            __tablename__ = "tb_relation_info"
            relation_type = Column(String(100), comment='关联类型', unique=False, nullable=False)
            point_1 = Column(String(100), comment='关联点1', unique=False, nullable=True)
            point_2 = Column(String(100), comment='关联点2', unique=False, nullable=True)

        self._RelationInfoPo = RelationInfoPo
        return self._RelationInfoPo

    @property
    def TmpSortInfoPo(self):
        if self._TmpSortInfoPo is not None:
            return self._TmpSortInfoPo

        class TmpSortInfoPo(self.BaseModel):
            """临时排序信息信息"""
            __tablename__ = "tmp_sort_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            unique_code = Column(String(100), comment='唯一码', unique=False, nullable=True, index=True)
            sort_value = Column(String(100), comment='排序值', unique=False, nullable=True)
            sort_index = Column(Integer, comment='排序索引', unique=False, nullable=True)

        self._TmpSortInfoPo = TmpSortInfoPo
        return self._TmpSortInfoPo

    @property
    def BlacklistInfoPo(self):
        if self._BlacklistInfoPo is not None:
            return self._BlacklistInfoPo

        class BlacklistInfoPo(self.BaseModel):
            """黑名单信息"""
            __tablename__ = "tb_blacklist_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            type_value = Column(String(255), comment='类型值', unique=False, nullable=True)

        self._BlacklistInfoPo = BlacklistInfoPo
        return self._BlacklistInfoPo

    @property
    def SitemapInfoPo(self):
        if self._SitemapInfoPo is not None:
            return self._SitemapInfoPo

        class SitemapInfoPo(self.BaseModel):
            """站点地图信息"""
            __tablename__ = "tb_sitemap_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            loc_url = Column(String(500), comment='url地址', unique=False, nullable=True)
            # 0.0 ~ 1.0
            priority = Column(Float, comment='优先权', unique=False, nullable=True)
            last_mod = Column(String(100), comment='最新一次更新时间(YYYY-MM-DD或，YYYY-MM-DDThh:mm:ss)',
                              unique=False, nullable=True)
            change_freq = Column(Enum(SitemapFreqEnum), comment='更新频率',
                                 default=SitemapFreqEnum.daily, unique=False, nullable=True)
            parent_id = Column(BigInteger, comment='主键', unique=False, nullable=True)

        self._SitemapInfoPo = SitemapInfoPo
        return self._SitemapInfoPo



"""
阿里云盘签到工具
pip install aligo -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""

from aligo import Aligo


class SignInAligo(Aligo):
    def sign_in_list(self):
        return self.post(
            '/v1/activity/sign_in_list',
            host='https://member.aliyundrive.com',
            body={'isReward': True},
            params={'_rx-s': 'mobile'}
        )

    def sign_in_reward(self, day):
        return self.post(
            '/v1/activity/sign_in_reward',
            host='https://member.aliyundrive.com',
            body={'signInDay': day},
            params={'_rx-s': 'mobile'}
        )

    def sign_in_festival(self):
        return self.post(
            '/v1/activity/sign_in_list',
            host='https://member.aliyundrive.com',
            body={},
            params={'_rx-s': 'mobile'}
        )


def get_sign_in_list(alipan_sign_api: SignInAligo) -> int:
    """获取签到列表"""
    log = alipan_sign_api._auth.log
    # 获取签到列表
    resp = alipan_sign_api.sign_in_list()
    result = resp.json()['result']
    signInCount = result['signInCount']
    log.info("本月签到次数: %d", signInCount)
    return signInCount


def do_sign_in(alipan_sign_api: SignInAligo):
    """执行签到"""
    log = alipan_sign_api._auth.log
    # 获取签到列表
    resp = alipan_sign_api.sign_in_list()
    result = resp.json()['result']
    signInCount = result['signInCount']
    log.info("本月签到次数: %d", signInCount)
    sign_in_resp = alipan_sign_api.sign_in_reward(signInCount)
    if sign_in_resp.status_code==200:
        log.info(sign_in_resp.json()['result']['notice'])
    else:
        log.error(sign_in_resp.text)


if __name__ == '__main__':
    ali = SignInAligo()
    # get_sign_in_list(ali)
    do_sign_in(ali)



"""
xpath工具： 安装：pip install lxml -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
html_tree.xpath('//div')
    . 本级，表示当前节点
    ..  上一级
    // 任意节点
    /  直接子节点
    *  任意元素  如： //*  选取文档中的所有元素
    //div[@class="A"]//text()  选择分区A中的所有文本
    //div[@class="A"]//text()[normalize-space()]  过滤掉空白字符串
    [@*]  获取带有属性的标签
    [@属性名]  获取带有某个属性的标签
    [@属性名="属性值"]  获取带有某个属性值的标签
    [contains(@属性名,"属性值")]  获取带有某个属性包含某个属性值的标签， 如：//a[contains(text(),"闻")]  或  //a[contains(@class,'active')]  class样式包含active
    [contains(.,"测试")]  标签包含某个值
    [starts‐with(@属性名,"开头值")]  开头匹配, 如： //a[starts-with(text(),"新")]
    [substring(@属性名,1)="data"]  获取带有某个属性的值的结尾子字符串是data的标签
    [@属性名1="属性值1" and @属性名2="属性值2"]  获取带有属性值1和属性值2的标签
    [name(.)!="a"]  排除某个标签
    a[last()]  获取a标签最后一个元素
    [position()> 1] 和 [position()< last()]  忽略xpath中的第一个和最后一个元素
    [text()="文本值"]  文本值等于某个值
    [text()=" 楼" and text()="栋" and text()="数： "]   多标签分隔字符串查找
    [text()[normalize-space()]  去除字符串空格换行
    //div[@class="tr-line clearfix"]/div/div[contains(text(),"层")]/text()  模糊查找标签内容字符保护层的元素
    /following-sibling::div  获取弟节点div
    /preceding-sibling::div  获取兄节点div
    /parent::div  获取父节点div
    [not(@class or @id)] 不包含某个属性
    [not(@class)] 不包含, 如： //tbody/tr[not(@class)]  排除有class属性标签
    [not(contains(@class, "disabled"))]  排除某个class属性标签

    //table/tr[not(@class=“tbhead”) and @class=“head"]  匹配所有的tr中不包含 tbhead 属性 和包含 head 的tr标签
    string(//p)  返回标签内包含字符串
    xpath("//tr[6]/td[2]/text() | //tr[7]/td[2]/text()")  在一个xpath中写的多个表达式用 `|` 分开， 每个表达式互不干扰
"""

from lxml import etree
from lxml.etree import _Element as Element, _ElementUnicodeResult as ElementUnicodeResult


def get_html_tree(html_file: str) -> Element:
    """转换html文件为html_tree"""
    parser = etree.HTMLParser(encoding="utf-8")
    return etree.parse(html_file, parser=parser)


def parse_to_html_tree(html_text: str) -> Element:
    """转换字符串为html_tree"""
    return etree.HTML(html_text)


def get_first(html_tree, xpath: str) -> Element | ElementUnicodeResult:
    """获取第一个元素"""
    result_list = html_tree.xpath(xpath)
    if result_list:
        return result_list[0]


def get_list(html_tree, xpath: str) -> list[Element] | list[ElementUnicodeResult]:
    """获取元素列表"""
    return html_tree.xpath(xpath)


def to_string(element: Element, only_text: bool = False,
              pretty_print: bool = False, encoding: str = 'utf8',
              with_comments: bool = True) -> str | None:
    """
    元素转为字符串
    :param element: xpath元素
    :param only_text: 是否只打印文本内容，默认是打印html代码
    :param pretty_print: 是否漂亮的打印
    :param encoding: 编码
    :param with_comments: 是否附带注释
    :return:
    """
    if element is None:
        return
    if only_text:
        return element.text
    return etree.tostring(element, pretty_print=pretty_print, encoding=encoding, method='html',
                          with_comments=with_comments).decode()



"""
微信小程序的api调用服务
"""
import requests

from afeng_tools.http_tool.decorator.http_auto_retry_tools import auto_retry
from afeng_tools.weixin_min_program import weixin_mp_settings
from afeng_tools.weixin_min_program.core.weixin_mp_models import TokenResult, Jscode2sessionResult, CheckResult, \
    GenerateUrlLink
from afeng_tools.weixin_min_program.model.weixin_mp_enums import WeixinMpConfigKeyEnum


@auto_retry(max_retries=3)
def request_jscode2session(mp_app_id: str, js_code: str) -> Jscode2sessionResult:
    """
    小程序登录： https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/user-login/code2Session.html
    :param mp_app_id:
    :param js_code:
    :return:
    """
    mp_app_id = weixin_mp_settings.get_config(mp_app_id, WeixinMpConfigKeyEnum.mp_app_id)
    mp_app_secret = weixin_mp_settings.get_config(mp_app_id, WeixinMpConfigKeyEnum.mp_app_secret)
    params = {
        'appid': mp_app_id,
        'secret': mp_app_secret,
        'js_code': js_code,
        'grant_type': 'authorization_code',
    }
    resp = requests.get('https://api.weixin.qq.com/sns/jscode2session', params=params)
    if resp.status_code == 200:
        return Jscode2sessionResult(**resp.json())


def request_token(mp_app_id: str) -> TokenResult:
    """
    获取接口调用凭据: https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/mp-access-token/getAccessToken.html
    :return:
    """
    mp_app_id = weixin_mp_settings.get_config(mp_app_id, WeixinMpConfigKeyEnum.mp_app_id)
    mp_app_secret = weixin_mp_settings.get_config(mp_app_id, WeixinMpConfigKeyEnum.mp_app_secret)
    params = {
        'appid': mp_app_id,
        'secret': mp_app_secret,
        'grant_type': 'client_credential',
    }
    resp = requests.get('https://api.weixin.qq.com/cgi-bin/token', params=params)
    if resp.status_code == 200:
        return TokenResult(**resp.json())


def request_check_session(access_token: str, openid: str, signature: str) -> CheckResult:
    """
    检验登录态： https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/user-login/checkSessionKey.html
    :return:
    """
    params = {
        'access_token': access_token,
        'openid': openid,
        'signature': signature,
        'sig_method': 'hmac_sha256'
    }
    resp = requests.get('https://api.weixin.qq.com/wxa/checksession', params=params)
    if resp.status_code == 200:
        return CheckResult(**resp.json())


def request_generate_url_link(access_token: str, page_path: str, query: str = None,
                              expire_interval: int = 30,
                              env_version: str = 'release') -> GenerateUrlLink:
    """
    获取URLLink: https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/qrcode-link/url-link/generateUrlLink.html
    :param access_token: 接口调用凭证
    :param page_path: 通过 URL Link 进入的小程序页面路径，必须是已经发布的小程序存在的页面，不可携带 query 。path 为空时会跳转小程序主页
    :param query: 通过 URL Link 进入小程序时的query，最大1024个字符，只支持数字，大小写英文以及部分特殊字符：!#$&'()*+,/:;=?@-._~%
    :param expire_interval: 到期失效的URL Link的失效间隔天数。生成的到期失效URL Link在该间隔时间到达前有效。最长间隔天数为30天。expire_type 为 1 必填
    :param env_version: 默认值"release"。要打开的小程序版本。正式版为 "release"，体验版为"trial"，开发版为"develop"，仅在微信外打开时生效。
    :return:
    """
    params = {
        'path': page_path,
        'expire_type': 1,
        'expire_interval': expire_interval,
        'env_version': env_version
    }
    if query:
        params['query'] = query
    api_url = 'https://api.weixin.qq.com/wxa/generate_urllink?access_token='+access_token
    resp = requests.post(api_url, params=params)
    if resp.status_code == 200:
        return GenerateUrlLink(**resp.json())




"""
GZIP压缩工具
"""
import gzip


def run_compress(input_filename: str, output_filename: str):
    with open(input_filename, 'rb') as f_in:
        with gzip.open(output_filename, 'wb') as f_out:
            f_out.writelines(f_in)


def run_decompress(input_filename: str, output_filename: str):
    with gzip.open(input_filename, 'rb') as f_in:
        with open(output_filename, 'wb') as f_out:
            f_out.writelines(f_in)


if __name__ == '__main__':
    # 压缩
    run_compress('example.txt', 'example.txt.gz')
    # 解压
    run_decompress('example.txt.gz', 'example_decompressed.txt')



import bz2


def run_compress(bz2_filename: str, input_file: str):
    """
    进行压缩，
    :param bz2_filename: 压缩后的bz2文件，如：example.txt.bz2
    :param input_file: 源文件，example.txt
    :return:
    """
    # 打开输入文件并读取内容
    with open(input_file, 'rb') as f_in:
        data = f_in.read()
    # 使用bz2压缩数据
    compressed_data = bz2.compress(data+b'\nafeng')
    # 将压缩数据写入输出文件
    with open(bz2_filename, 'wb') as f_out:
        f_out.write(compressed_data)


def run_decompress(bz2_filename: str, output_file: str):
    """
    解压
    :param bz2_filename: 压缩后的bz2文件，如：example.txt.bz2
    :param output_file: 解压后文件，example.txt
    :return:
    """
    # 打开压缩文件并读取内容
    with open(bz2_filename, 'rb') as f_in:
        compressed_data = f_in.read()
    # 使用bz2解压缩数据
    decompressed_data = bz2.decompress(compressed_data)
    # 将解压缩数据写入输出文件
    with open(output_file, 'wb') as f_out:
        f_out.write(decompressed_data)


if __name__ == '__main__':
    # 使用压缩函数
    run_compress('example.txt.bz2', 'example.txt')
    # 使用解压缩函数
    run_decompress('example.txt.bz2', 'example_decompressed.txt')



import asyncio
import traceback
from typing import Callable

from fastapi import FastAPI
from fastapi.exceptions import RequestValidationError
from fastapi.logger import logger
from starlette.requests import Request
from starlette.responses import HTMLResponse

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.exception_tools.common_exception import AfengException, HttpException
from afeng_tools.fastapi_tool import fastapi_response_tools, fastapi_settings, fastapi_request_tools
from afeng_tools.fastapi_tool.core.fastapi_enums import FastapiConfigKeyEnum
from afeng_tools.fastapi_tool.template.item import Error404DataItem, Error501DataItem, Error500DataItem


class HtmlFileResponseException(Exception):
    """html文件响应异常：用于抛异常方式响应html"""
    def __init__(self, html_file: str, binary_flag: bool = False):
        self.html_file = html_file
        self.binary_flag = binary_flag


class HtmlResponseException(Exception):
    """html响应异常：用于抛异常方式响应html"""
    def __init__(self, html_code: str, ):
        self.html_code = html_code


def register_exception_handler(app: FastAPI, app_dict: dict[str, AppInfo] = None):
    """注册捕获全局异常"""

    @app.exception_handler(HtmlResponseException)
    async def html_response_error_handler(request: Request, html_response: HtmlResponseException) -> HTMLResponse:
        """
        通过raise响应html
        :param request: 请求头信息
        :param html_response: html响应
        :return:
        """
        return fastapi_response_tools.resp_html(html_code=html_response.html_code)

    @app.exception_handler(HtmlFileResponseException)
    async def html_file_response_error_handler(request: Request,
                                               html_file_response: HtmlFileResponseException) -> HTMLResponse:
        """
        通过raise响应html
        :param request: 请求头信息
        :param html_file_response: html文件响应
        :return:
        """
        return fastapi_response_tools.resp_html_file(html_file=html_file_response.html_file,
                                                     binary_flag=html_file_response.binary_flag)

    @app.exception_handler(RequestValidationError)
    async def request_validation_exception_handler(request: Request, exc: RequestValidationError):
        """
        请求参数验证异常
        :param request: 请求头信息
        :param exc: 异常对象
        :return:
        """
        app_info = fastapi_request_tools.get_request_app_info(request, app_dict)
        # 日志记录异常详细上下文
        logger.error(
            f"[{request.url}]全局异常: \n{request.method}URL{request.url}\nHeaders:{request.headers}\n{traceback.format_exc()}")
        message = '缺少必要的参数，请检查您的输入！'
        if exc.args and isinstance(exc.args, list):
            message = '，'.join(exc.args)
        return fastapi_response_tools.resp_500(error_data=Error500DataItem(sub_message=message), request=request,
                                               app_info=app_info)

    @app.exception_handler(AfengException)
    async def afeng_exception_handler(request: Request, exc: AfengException):
        """
        自定义异常处理
        :param request: 请求头信息
        :param exc: 异常对象
        :return:
        """
        app_info = fastapi_request_tools.get_request_app_info(request, app_dict)
        # 日志记录异常详细上下文
        logger.error(
            f"[{request.url}]全局异常: \n{request.method}URL{request.url}\nHeaders:{request.headers}\n{traceback.format_exc()}")
        return fastapi_response_tools.resp_501(
            error_data=Error501DataItem(message=exc.message, sub_message=exc.sub_message), request=request,
            app_info=app_info)

    @app.exception_handler(HttpException)
    async def http_exception_handler(request: Request, exc: HttpException):
        """
        http异常处理
        :param request: 请求头信息
        :param exc: 异常对象
        :return:
        """
        app_info = fastapi_request_tools.get_request_app_info(request, app_dict)
        # 日志记录异常详细上下文
        logger.error(
            f"[{request.url}]全局异常: \n{request.method}URL{request.url}\nHeaders:{request.headers}\n{traceback.format_exc()}")
        if exc.status_code == 404:
            return fastapi_response_tools.resp_404(
                error_data=Error404DataItem(message=exc.message, sub_message=exc.sub_message), request=request,
                app_info=app_info)
        elif exc.status_code == 501:
            return fastapi_response_tools.resp_501(
                error_data=Error501DataItem(message=exc.message, sub_message=exc.sub_message), request=request,
                app_info=app_info)
        return fastapi_response_tools.resp_500(
            error_data=Error500DataItem(message=exc.message, sub_message=exc.sub_message), request=request,
            app_info=app_info)

    @app.exception_handler(Exception)
    async def all_exception_handler(request: Request, exc: Exception):
        """
        全局所有异常
        :param request:
        :param exc:
        :return:
        """
        traceback_msg = traceback.format_exc()
        app_info = fastapi_request_tools.get_request_app_info(request, app_dict)
        if app_info and hasattr(app_info, 'error500_background_work_func') and app_info.error500_background_work_func:
            background_work_func = app_info.error500_background_work_func
        else:
            background_work_func = fastapi_settings.get_config(FastapiConfigKeyEnum.error500_background_work_func)
        if background_work_func and isinstance(background_work_func, Callable):
            asyncio.ensure_future(background_work_func(request, exc, traceback_msg))
        logger.error(
            f"全局异常: \n{request.method}URL:{request.url}\nHeaders:{request.headers}\n{traceback_msg}", exc)
        return fastapi_response_tools.resp_500(error_data=Error500DataItem(), request=request, app_info=app_info)



from afeng_tools.weixin_tool.core.model.item.wx_receive_event_models import WeixinEventItem
from afeng_tools.weixin_tool.core.model.item.wx_receive_msg_models import WeixinTextMsgItem, WeixinImageMsgItem, \
    WeixinVoiceMsgItem, WeixinVideoMsgItem, WeixinLocationMsgItem, WeixinLinkMsgItem
from afeng_tools.weixin_tool.core.parse.receive_event import parse_event
from afeng_tools.weixin_tool.core.parse.receive_msg import parse_msg
from afeng_tools.weixin_tool.core.parse.receive_msg.parse_tool import parse_msg_type


def parse(openid,
          msg_bytes) -> WeixinTextMsgItem | WeixinImageMsgItem | WeixinVoiceMsgItem | WeixinVideoMsgItem | WeixinLocationMsgItem | WeixinLinkMsgItem | WeixinEventItem:
    """转换接收到的微信消息"""
    msg = msg_bytes.decode('utf-8')
    msg_type = parse_msg_type(msg)
    if msg_type == 'text':
        return parse_msg.parse_text_msg(openid, msg)
    if msg_type == 'image':
        return parse_msg.parse_image_msg(openid, msg)
    if msg_type == 'voice':
        return parse_msg.parse_voice_msg(openid, msg)
    if msg_type == 'video' or msg_type == 'shortvideo':
        return parse_msg.parse_video_msg(openid, msg)
    if msg_type == 'location':
        return parse_msg.parse_location_msg(openid, msg)
    if msg_type == 'link':
        return parse_msg.parse_link_msg(openid, msg)
    if msg_type == 'event':
        return parse_event.parse(openid, msg)



"""
临时文件工具
"""
import tempfile
from os import PathLike

from afeng_tools.decorator_tool import decorator_tools


def auto_tmp_file(func):
    """自动注入临时文件：tmp_file. 当执行结束，文件会自动删除"""

    def wrap(*args, **kwargs):
        if 'tmp_file' in kwargs:
            return decorator_tools.run_func(func, *args, **kwargs)
        with tempfile.TemporaryFile() as fd:
            kwargs['tmp_file'] = fd
            return decorator_tools.run_func(func, *args, **kwargs)


def auto_tmp_dir(func):
    """自动注入临时目录：tmp_dir. 当执行结束，目录会自动删除"""

    def wrap(*args, **kwargs):
        if 'tmp_dir' in kwargs:
            return decorator_tools.run_func(func, *args, **kwargs)
        with tempfile.TemporaryDirectory() as tmp_dir_name:
            kwargs['tmp_dir'] = tmp_dir_name
            return decorator_tools.run_func(func, *args, **kwargs)


def write_line(tmp_file, line_list: list[str]):
    """写入行"""
    for tmp_line in line_list:
        tmp_file.write(f'{tmp_line}\n'.encode('utf-8'))


def read_line(tmp_file) -> list[str]:
    """读取行"""
    # 将文件指针移到开始处，准备读取文件
    tmp_file.seek(0)
    return [tmp_line.decode('utf-8') for tmp_line in tmp_file.readlines()]


def get_user_tmp_dir() -> str:
    """获取用户的临时目录"""
    return tempfile.gettempdir()


def create_tmp_path(suffix: str = None, prefix: str = None) -> str:
    """创建临时路径"""
    return tempfile.mkdtemp(prefix=prefix, suffix=suffix)


if __name__ == '__main__':
    print(get_user_tmp_dir())



from typing import Any

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService
from afeng_tools.fastapi_tool.common.service import icon_base_service
from afeng_tools.pydantic_tool.model.common_models import LinkItem


class GroupPoService(PoService):
    """
    使用示例：group_po_service = GroupPoService(app_info.db_code, GroupInfoPo)
    """
    _table_name_ = "tb_group_info"

    def get_group(self, title: str, group_code: str, type_code: str) -> Any:
        group_po = self.get(self.model_type.title == title,
                            self.model_type.code == group_code, self.model_type.type_code == type_code)
        if not group_po:
            group_po = self.save(self.model_type(
                title=title,
                code=group_code,
                type_code=type_code
            ), auto_code=True)
        return group_po

    def get_by_code(self, group_code: str) -> Any:
        return self.get(self.model_type.code == group_code)

    @classmethod
    def convert_to_link_item(cls, group_po, is_active: bool = False, group_href_prefix: str = '/group') -> LinkItem:
        return LinkItem(
            target='_blank',
            title=group_po.title,
            href=f'{group_href_prefix}/{group_po.code}',
            code=group_po.code,
            description=group_po.description,
            image=icon_base_service.get_icon_code(icon_type=group_po.icon_type,
                                                  icon_value=group_po.icon_value,
                                                  alt=group_po.title,
                                                  image_src=group_po.image_src),
            is_active=is_active)



import os
from typing import List, Callable

from aligo import Aligo, CreateFileResponse, BaseFile
from aligo.types.Enum import CheckNameMode


def upload_files(alipan_api: Aligo, local_file_paths: List[str],
                 parent_file_id: str = 'root',
                 drive_id: str = None,
                 check_name_mode: CheckNameMode = "auto_rename") -> List[BaseFile]:
    """
    批量上传文件
    :param alipan_api:
    :param local_file_paths: [List[str]] 文件路径列表
    :param parent_file_id: Optional[str] 父文件夹id, 默认为 'root'
    :param drive_id: Optional[str] 指定网盘id, 默认为 None
    :param check_name_mode: Optional[CheckNameMode] 检查文件名模式, 默认为 'auto_rename'
    :return: [List[BaseFile]]
    """
    return alipan_api.upload_files(file_paths=local_file_paths,
                                   parent_file_id=parent_file_id, drive_id=drive_id,
                                   check_name_mode=check_name_mode)


def upload_folder(alipan_api: Aligo,
                  local_folder_path: str,
                  parent_file_id: str = 'root',
                  drive_id: str = None,
                  check_name_mode: CheckNameMode = "auto_rename",
                  folder_check_name_mode: CheckNameMode = 'refuse',
                  file_filter: Callable[[os.DirEntry], bool] = lambda x: False) -> List:
    """
    上传文件夹
    :param alipan_api: Aligo
    :param local_folder_path: [str] 文件夹路径
    :param parent_file_id: Optional[str] 父文件夹id, 默认为 'root'
    :param drive_id: [str] 指定网盘id, 默认为 None, 如果为 None, 则使用默认网盘
    :param check_name_mode: [CheckNameMode] 检查文件名模式, 默认为 'auto_rename'
    :param folder_check_name_mode: [CheckNameMode] 检查文件夹名模式, 默认为 'refuse'
    :param file_filter: 文件过滤函数, 如果返回True，则忽略上册该文件
    :return: [List]
    """
    return alipan_api.upload_folder(folder_path=local_folder_path,
                                    parent_file_id=parent_file_id,
                                    drive_id=drive_id,
                                    check_name_mode=check_name_mode,
                                    folder_check_name_mode=folder_check_name_mode,
                                    file_filter=file_filter)




import json
import os.path
from typing import Any, Optional, Mapping, Sequence

from pydantic import BaseModel
from starlette.background import BackgroundTask
from starlette.requests import Request
from starlette.responses import FileResponse, Response, RedirectResponse, HTMLResponse, PlainTextResponse

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool.common.service import fastapi_error_service
from afeng_tools.fastapi_tool.core.fastapi_response import json_resp
from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response, create_cache_template_response, \
    create_template_response_with_cache
from afeng_tools.fastapi_tool.template.item import Error404DataItem, Error501DataItem, Error500DataItem
from afeng_tools.file_tool import file_tools
from afeng_tools.http_tool import http_request_tools
from afeng_tools.sqlalchemy_tools.core.sqlalchemy_base_model import is_model_instance
from afeng_tools.sqlalchemy_tools.tool import sqlalchemy_model_tools


def resp_404(error_data: Error404DataItem, request: Request = None, context_data: dict = None,
             app_info: AppInfo = None):
    return fastapi_error_service.handle_404(error_data=error_data, request=request, context_data=context_data,
                                            app_info=app_info)


def resp_501(error_data: Error501DataItem, request: Request = None, context_data: dict = None,
             app_info: AppInfo = None):
    return fastapi_error_service.handle_501(error_data=error_data, request=request, context_data=context_data,
                                            app_info=app_info)


def resp_500(error_data: Error500DataItem, request: Request = None, context_data: dict = None,
             app_info: AppInfo = None):
    return fastapi_error_service.handle_500(error_data=error_data, request=request, context_data=context_data,
                                            app_info=app_info)


def resp_template(request: Request, template_file: str, context_data: dict[str, Any],
                  is_redirect: bool = False, redirect_name: str = None,
                  cache_html_file: str = None):
    """响应模板"""
    if is_redirect:
        return create_cache_template_response(request=request, template_file=template_file,
                                              context=context_data, cache_html_name=redirect_name)
    return create_template_response_with_cache(request=request,
                                               template_file=template_file,
                                               context=context_data,
                                               cache_html_file=cache_html_file)


def resp_json(data: Any = None, error_no: int = 0, message: str | Sequence = 'success', app_info: AppInfo = None):
    if is_model_instance(data) or (data and isinstance(data, list) and len(data) > 0 and is_model_instance(data[0])):
        data = json.loads(sqlalchemy_model_tools.to_json(data))
    if isinstance(data, BaseModel):
        data = data.model_dump(mode='json')
    return json_resp(error_no=error_no, message=message, data=data)


def resp_file(file_path: str, file_name: str = None, download_flag: bool = False,
              context_data: dict = None, app_info: AppInfo = None) -> Response:
    """响应文件"""
    if not os.path.exists(file_path):
        return resp_404(error_data=Error404DataItem(), context_data=context_data, app_info=app_info)
    response = FileResponse(file_path)
    with open(file_path, "rb") as file:
        if download_flag:
            if file_name is None:
                file_name = os.path.split(file_path)[1]
            response.headers["Content-Disposition"] = f"attachment; filename={file_name}"
        response.body = file.read()
        return response


def redirect(target_url: str, status_code: int = 307,
             headers: Optional[Mapping[str, str]] = None,
             background: Optional[BackgroundTask] = None) -> RedirectResponse:
    """重定向"""
    return RedirectResponse(target_url, status_code=status_code, headers=headers, background=background)


def resp_html_file(html_file: str, binary_flag: bool = False) -> HTMLResponse:
    """响应html文件"""
    return HTMLResponse(content=file_tools.read_file(html_file, binary_flag=binary_flag))


def resp_html(html_code: str) -> HTMLResponse:
    """响应html内容"""
    return HTMLResponse(content=html_code)


def resp_text(text: str) -> PlainTextResponse:
    """响应文本内容"""
    return PlainTextResponse(content=text)


def resp_not_found(request: Request, message: str = '资源'):
    """响应找不到"""
    sub_message = f'您访问的{message}不存在或已被删除！ (｡•ˇ‸ˇ•｡)'
    if http_request_tools.is_json(request.headers):
        return json_resp(error_no=404, message=sub_message, http_status=404)
    return resp_404(error_data=Error404DataItem(
        message=f'很抱歉，找不到{message}！',
        sub_message=sub_message
    ), request=request)




from datetime import datetime
from typing import Callable

from starlette.requests import Request
from starlette.responses import Response, PlainTextResponse

from afeng_tools.fastapi_tool.core.fastapi_http_cacher import HttpCacher


class FastapiHttpMiddleware:
    def __init__(self, request: Request, async_next_callback: Callable[[Request, ], Response],
                 http_cacher_chain: list[HttpCacher],
                 is_debug: bool = False,
                 has_before_print: bool = True, is_cache: bool = False):
        self.request = request
        self.async_next_callback = async_next_callback
        self.is_debug = is_debug
        self.has_before_print = has_before_print
        self.is_cache = is_cache
        self.http_cacher_chain = http_cacher_chain
        self.response = None

    def before_print(self):
        """响应前输出"""
        real_ip = ''
        if self.request.headers:
            real_ip = self.request.headers.get('X-Real-IP')
        current_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        print(f"[{current_time}][{real_ip}-{self.request.method}]{self.request.url}")

    def is_in_blacklist(self) -> bool:
        """是否在黑名单中"""
        return False

    @classmethod
    def resp_forbidden(cls, http_status: int = 403) -> Response:
        """响应禁止"""
        return PlainTextResponse('Access denied', status_code=http_status)

    async def resp(self):
        """响应"""
        return await self.async_next_callback(self.request)

    async def run(self) -> Response:
        """运行中间件拦截"""
        if self.has_before_print:
            self.before_print()
        # 是否不是调试
        if not self.is_debug:
            # 是否在黑名单中
            if self.is_in_blacklist():
                return self.resp_forbidden()
            # 是否缓存
            if self.is_cache:
                for tmp_http_cacher in self.http_cacher_chain:
                    if tmp_http_cacher.is_need_cache():
                        has_cache, cache_response = tmp_http_cacher.get_cache()
                        if has_cache:
                            return cache_response

                self.response = await self.resp()
                if not self.request.scope.get('request_not_cache'):
                    for tmp_http_cacher in self.http_cacher_chain:
                        resp = await tmp_http_cacher.save_cache(self.response)
                        if resp:
                            return resp
                return self.response
        return await self.resp()


from starlette.requests import Request
from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.fastapi_tool.common.service.error_base_service import ErrorService
from afeng_tools.fastapi_tool.core.fastapi_response import json_resp
from afeng_tools.fastapi_tool.template.item import Error501DataItem, Error404DataItem, Error500DataItem
from afeng_tools.import_tool import import_tools


def _import_error_service(request: Request = None, app_info: AppInfo = None):
    app_info = app_info if app_info else (request.scope.get('app_info') if request else None)
    if app_info:
        if app_info.error_service_class:
            return import_tools.import_class(app_info.error_service_class)


def handle_404(error_data: Error404DataItem, request: Request = None, context_data: dict = None,
               app_info: AppInfo = None):
    Error_Service = _import_error_service(request=request, app_info=app_info)
    if Error_Service and issubclass(Error_Service, ErrorService):
        return Error_Service().handle_404(error_data=error_data, request=request, context_data=context_data,
                                          app_info=app_info)
    return json_resp(error_no=404, message=error_data.message, sub_message=error_data.sub_message)


def handle_500(error_data: Error500DataItem, request: Request = None, context_data: dict = None,
               app_info: AppInfo = None):
    Error_Service = _import_error_service(request=request, app_info=app_info)
    if Error_Service and issubclass(Error_Service, ErrorService):
        return Error_Service().handle_500(error_data=error_data, request=request, context_data=context_data,
                                          app_info=app_info)
    return json_resp(error_no=500, message=error_data.message, sub_message=error_data.sub_message)


def handle_501(error_data: Error501DataItem, request: Request = None, context_data: dict = None,
               app_info: AppInfo = None):
    Error_Service = _import_error_service(request=request, app_info=app_info)
    if Error_Service and issubclass(Error_Service, ErrorService):
        return Error_Service().handle_501(error_data=error_data, request=request, context_data=context_data,
                                          app_info=app_info)
    return json_resp(error_no=501, message=error_data.message, sub_message=error_data.sub_message)



from typing import List, Callable

from aligo import Aligo, BaseFile

from afeng_tools.datetime_tool.datetime_tools import one_minute_seconds
from .core import alipan_file_tools
from .core.custom_aligo import CustomAligo


def get_download_url(alipan_api: Aligo, file_id: str,
                     file_name: str = None,
                     expire_seconds: int = 30 * one_minute_seconds,
                     drive_id: str = None) -> str:
    """
    获取下载链接
    :param alipan_api:
    :param file_id: 文件 id
    :param file_name: 文件名
    :param expire_seconds: 下载链接有效时间, 默认为半小时（30分钟）, 允许的最大值是4小时
    :param drive_id:
    :return:
    """
    download_url_resp = alipan_api.get_download_url(file_id=file_id, file_name=file_name,
                                                    expire_sec=expire_seconds,
                                                    drive_id=drive_id)
    return download_url_resp.url


def batch_get_download_url(alipan_api: Aligo,
                           file_id_list: List[str],
                           expire_seconds: int = 30 * one_minute_seconds,
                           drive_id=None) -> dict[str, str]:
    """
    批量获取下载链接
    :param alipan_api:
    :param file_id_list:
    :param expire_seconds:
    :param drive_id:
    :return: {file_id: 下载链接}
    """
    resp_list = alipan_api.batch_download_url(file_id_list=file_id_list,
                                              expire_sec=expire_seconds,
                                              drive_id=drive_id)
    return {tmp_resp.body.file_id: tmp_resp.body.url for tmp_resp in resp_list if tmp_resp.body}


def download_folder(alipan_api: Aligo, folder_file_id: str, local_folder: str = '.',
                    drive_id: str = None,
                    file_filter: Callable[[BaseFile], bool] = lambda x: False) -> str:
    """
    下载文件夹
    :param alipan_api:
    :param folder_file_id: 文件夹 id
    :param local_folder: 本地文件夹路径, 默认为当前目录, 即下载到哪里
    :param drive_id:  文件夹所在的网盘 id
    :param file_filter: 文件过滤函数，如果返回True，则该文件不下载
    :return: 本地文件夹路径
    """
    return alipan_api.download_folder(folder_file_id=folder_file_id,
                                      local_folder=local_folder, drive_id=drive_id,
                                      file_filter=file_filter)


def download_file_by_url(alipan_api: Aligo, download_url: str, local_file_path: str) -> str:
    """
    根据下载地址下载文件
    :param alipan_api:
     :param download_url: 下载地址
    :param local_file_path: 文件路径(包含文件名)
    :return: 本地文件路径
    """
    return alipan_api.download_file(file_path=local_file_path, url=download_url)


def download_file_by_id(alipan_api: Aligo, file_id: str, local_folder: str = '.') -> str:
    """
    根据下载地址下载文件
    :param alipan_api:
     :param file_id: 文件 id
    :param local_folder: 本地文件夹路径, 默认为当前目录
    :return: 本地文件路径
    """
    return download_file(alipan_api,
                         alipan_file=alipan_api.get_file(file_id),
                         local_folder=local_folder)


def download_file_by_path(alipan_api: CustomAligo, pan_path: str = '/',
                          local_folder: str = '.',
                          parent_file_id: str = 'root') -> str:
    """
    下载文件/文件夹
    :param alipan_api: Aligo
    :param pan_path: 网盘路径(相对父文件)，如：我的资源/音乐
    :param local_folder: 本地存储路径，如：D:/阿里云盘
    :param parent_file_id: 父文件id
    :return: 本地文件路径
    """
    return download_file(alipan_api,
                         alipan_file=alipan_api.get_file_by_path(path=pan_path,
                                                                 parent_file_id=parent_file_id),
                         local_folder=local_folder)


def download_file(alipan_api: Aligo, alipan_file: BaseFile, local_folder: str = '.') -> str:
    """
    根据下载地址下载文件
    :param alipan_api:
     :param alipan_file: 文件对象
    :param local_folder: 本地文件夹路径, 默认为当前目录
    :return: 本地文件路径
    """
    if alipan_file_tools.is_folder(alipan_file):
        return download_folder(alipan_api, folder_file_id=alipan_file.file_id,
                               local_folder=local_folder, drive_id=alipan_file.drive_id)
    return alipan_api.download_file(file=alipan_file, local_folder=local_folder)






import os
from abc import ABCMeta, abstractmethod
from playwright.sync_api import Page

from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.file_tool import tmp_file_tools
from afeng_tools.log_tool import loguru_tools
from afeng_tools.playwright_tool import http_header_tools, http_cookie_tools
from afeng_tools.playwright_tool.decorator.playwright_decorators import auto_input

logger = loguru_tools.get_logger()


class BaseAutoSpider(metaclass=ABCMeta):
    """自动化爬虫基类"""
    # 爬虫编码
    spider_code: str = None
    # 是否headless
    headless: bool = False

    def __init__(self):
        if not self.spider_code:
            self.spider_code = hashlib_tools.calc_md5(__file__)
        auth_save_path = os.path.join(tmp_file_tools.get_user_tmp_dir(), 'spider_auth-file')
        os.makedirs(auth_save_path, exist_ok=True)
        self.cookie_file = os.path.join(auth_save_path, f'{self.spider_code}-cookie.txt')
        self.header_file = os.path.join(auth_save_path, f'{self.spider_code}-header.txt')

    def _refresh_auth(self, web_page: Page):
        """刷新认证"""
        logger.info('[登录]进入登录逻辑')
        self.login(web_page=web_page)
        logger.info('[登录]登录完成')
        logger.info('[保存]认证header')
        http_header_tools.save_headers(web_page, self.header_file)
        logger.info('[保存]认证cookie')
        http_cookie_tools.save_cookies(web_page, self.cookie_file)

    def _load_auth_file(self, web_page: Page):
        """加载认证文件"""
        if not os.path.exists(self.cookie_file) or not os.path.exists(self.header_file):
            self._refresh_auth(web_page=web_page)
        return self.header_file, self.cookie_file

    def load_auth(self, web_page: Page):
        """加载认证"""
        logger.info('加载认证信息')
        header_file, cookie_file = self._load_auth_file(web_page)
        http_header_tools.set_headers(page=web_page, auth_header_file=header_file)
        http_cookie_tools.set_cookies(page=web_page, auth_cookie_file=cookie_file)

    def login(self, web_page: Page):
        """登录逻辑：如果需要登录，则需要实现该方法"""
        raise NotImplementedError('没有继承实现登录逻辑，无法进行认证操作！')

    @abstractmethod
    @auto_input(headless=headless)
    def run(self, web_page: Page = None, **kwargs):
        pass



"""
FastAPI路由工具
"""
import importlib
import os

from fastapi import APIRouter
from starlette.routing import BaseRoute

from afeng_tools.application_tool.application_models import AppInfo


def create_router(prefix: str, tags: list[str], router_list: list[BaseRoute] = None) -> APIRouter:
    """创建路由"""
    if router_list:
        router = APIRouter(prefix=prefix, tags=tags, routes=router_list)
    else:
        router = APIRouter(prefix=prefix, tags=tags)
    return router


def router_add_prefix(prefix: str, router_list: list[APIRouter | BaseRoute]):
    """路由添加前缀"""
    result_router_list = []
    if prefix:
        for tmp_router in router_list:
            tmp_router.prefix = f'{prefix}{tmp_router.prefix}'
            if tmp_router.routes:
                for child_router in tmp_router.routes:
                    child_router.path = f'{prefix}{child_router.path}'
            result_router_list.append(tmp_router)
    return result_router_list


def auto_load_routers(web_root_path: str, web_root_name: str = None,
                      view_file_name: str = 'views', router_name='router') -> list[APIRouter | BaseRoute]:
    """自动加载web.{app}.views文件中的路由"""
    router_list = []
    if web_root_name is None:
        web_root_name = os.path.split(web_root_path)[1]
    if os.path.exists(web_root_path):
        for app_name in os.listdir(web_root_path):
            if os.path.isdir(
                    os.path.join(web_root_path, app_name)) and app_name != '__pycache__' and app_name != 'admin':
                if os.path.exists(os.path.join(web_root_path, app_name, view_file_name + '.py')):
                    web_app_views = importlib.import_module(f'{web_root_name}.{app_name}.{view_file_name}')
                    router_list.append(web_app_views.__getattribute__(router_name))
    return router_list


def auto_load_apps_router(app_info_dict: dict[str, AppInfo], apps_prefix: str = 'apps', web_path_name: str = 'web') -> \
list[APIRouter | BaseRoute]:
    """自动加载apps下app的router"""
    router_list = []
    for tmp_code, tmp_app_info in app_info_dict.items():
        if tmp_app_info.web_path and os.path.exists(tmp_app_info.web_path):
            router_list.extend(router_add_prefix(prefix=tmp_app_info.prefix,
                                                 router_list=auto_load_routers(tmp_app_info.web_path,
                                                                               web_root_name=f'{apps_prefix}.{tmp_code}.{web_path_name}')))
    return router_list



from afeng_tools.fastapi_tool.common.po_service.link_po_service_ import LinkPoService
from afeng_tools.fastapi_tool.common.service.base_service import BaseService


class LinkService(BaseService):
    """
    使用示例：link_service = LinkService(app_info.db_code, LinkInfoPo)
    """

    po_service_type = LinkPoService
    pass



import datetime

import json
from array import array
from enum import Enum
from pydantic.json import pydantic_encoder
from _decimal import Decimal

from afeng_tools.pydantic_tool.model.common_models import EnumItem


class JsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        elif isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        elif isinstance(obj, bytes):
            return str(obj, encoding='utf-8')
        elif isinstance(obj, int):
            return int(obj)
        elif isinstance(obj, float):
            return float(obj)
        elif isinstance(obj, array):
            return obj.tolist()
        elif isinstance(obj, Decimal):
            return str(obj)
        elif isinstance(obj, Enum):
            if isinstance(obj.value, str | int | float):
                return obj.value
            elif isinstance(obj.value, EnumItem):
                if obj.value.value is not None:
                    return obj.value.value
            return obj.name

        else:
            return super(JsonEncoder, self).default(obj)


def to_json(obj_data) -> str:
    """将Python中的对象转换为JSON中的字符串对象"""
    return json.dumps(obj_data, cls=JsonEncoder, ensure_ascii=False, indent=4)


def to_json_str(obj_data) -> str:
    """将Python中的对象转换为JSON中的字符串对象"""
    return json.dumps(obj_data, indent=4, ensure_ascii=False, default=pydantic_encoder)


def to_obj(str_data):
    """将JSON中的字符串对象转换为Python中的对象"""
    return json.loads(str_data)


def model_to_dict(model):
    """Model实例转dict"""
    model_dict = dict(model.__dict__)
    del model_dict['_sa_instance_state']
    return model_dict


def model_to_dict2(model):
    """单个对象转dict(效果等同上面的那个)"""
    return {c.name: getattr(model, c.name) for c in model.__table__.columns}


def model_to_json(model) -> str:
    """model或model集合转换为json字符串"""
    if isinstance(model, list):
        return to_json([model_to_dict2(tmp) for tmp in model])
    else:
        return to_json(model_to_dict(model))



import json

import requests

from afeng_tools.weixin_tool import weixin_settings
from afeng_tools.weixin_tool.core.weixin_enums import WeixinConfigKeyEnum
from afeng_tools.weixin_tool.custom_menu.wx_custome_menu_models import WeixinButtonItem, WeixinViewButtonItem, \
    WeixinMiniProgramButtonItem, WeixinScanWaitButtonItem
from afeng_tools.weixin_tool.token import wx_token_tools


def create_menu(access_token: str, menu_list: list):
    menu_dict_list = [tmp.model_dump() for tmp in menu_list]
    json_data = '{ "button": ' + json.dumps(menu_dict_list, ensure_ascii=False) + ' }'
    resp = requests.post(url=f'https://api.weixin.qq.com/cgi-bin/menu/create?access_token={access_token}',
                         json=json_data)
    if resp.status_code == 200:
        resp_json = resp.json()
        if resp_json.get('errcode'):
            print(f"[{resp_json.get('errcode')}]{resp_json.get('errmsg')}")


if __name__ == '__main__':
    menu_list = []
    menu_list.append(WeixinButtonItem(name='优秀网站',
                                      sub_button=[
                                          WeixinViewButtonItem(name='阿锋书屋', url='https://www.afengbook.com')]))

    menu_list.append(WeixinButtonItem(name='小程序',
                                      sub_button=[WeixinMiniProgramButtonItem(name='土味情话', appid='',
                                                                              pagepath='pages/home/index')]))
    menu_list.append(WeixinScanWaitButtonItem(name='扫一扫', key='scancode_10001'))
    access_token = wx_token_tools.get_access_token(weixin_settings.get_config(WeixinConfigKeyEnum.weixin_app_id),
                                                   weixin_settings.get_config(WeixinConfigKeyEnum.weixin_app_secret))
    create_menu(access_token, menu_list)
